function rho = density_n2sw(press,temp,sal,mN2)

% DENSITY_N2SW - calculates the density of a N2-seawater solution
%
%       Usage: rho = density_n2sw(p,T,S,m)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (0.1 < p < 60), 
%           T       - temperature [degC] (0 < T < 127),
%           S       - practical salinity (0 < S < 350)
%           m       - molality of dissolved N2 [mol/kg]
%                         (default value is saturation concentration) 
%                         (input as scalar or p x T x S matrix)
%
%       Output parameter: 
%           rho      - density of N2-rich seawater [kg/m^3]
%
%       References:
%       Mao & Duan (2006) Fluid Phase Equilibria 248, 103-114.
%
%       Comments:
%       The validity range for salinity is not specified and 
%       the limit for NaCl is taken instead.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (April 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_N2SW:Input','DENSITY_N2SW: Missing input parameter! Please provide p,T and S!')
else

if nargin == 3 % if no value for mn2 is provided, use saturation concentration
    
% call external function to calculate molality of dissolved N2 at saturation  
[mn2_sat] = solu_n2sw(press,temp,sal); % dissolved N2 at saturation [mol/kg]
mn2_sw = mn2_sat;

else
    mn2_sw = mN2;
end


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>60) && ~any(temp<0 | temp>127) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 60);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>127);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('DENSITY_N2SW:OutOfParameterRange','DENSITY_N2SW: Input values out of range!')
    warning('DENSITY_N2SW:OutOfParameterRange','Equations valid for p=0.1-60 MPa, T=0-127 degC, S=0-350.')
end

[P, T, S] = ndgrid(press*10,temp+273.15,sal); % pressure in [bar] temperature in [K]

R = 83.14472; % gas constant in bar*cm^3/mol/K 
MmN2 = 28; % molar mass of N2


% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK] = seasalt(S); % m [mol/kg]

% call external function to calculate density of seawater;
rsw = density_sw(press,temp,sal);
rho_sw = rsw./1000; % [g/cm^3]

C6m =-0.49542866d-3; 
C7m =0.12698747d-5;
C8m  =0.51411144;
C9m  =-0.64733978d-4; 

C6l = -0.13711527d-4;
C9l = 0.71037217d-5;

%------------------------------------------------------------------------
% calculation of partail molar volume of dissolved N2
%------------------------------------------------------------------------
dmRTdp = C6m + C7m*T + C8m./T + 2*C9m*P./T; % d(mu/(RT))/dp 
dlambdadp = C6l + 2*C9l*P./T; % d(lambda)/dp

VN2 = R.*T.*(dmRTdp + 2*(mNa + mK + 2*mCa + 2*mMg).*dlambdadp);

%------------------------------------------------------------------------
% Calculation of mass of solution per 1000 g water [g/(1000g water)]
%------------------------------------------------------------------------
Sa = S*1.005; % absolute salinity
m_sol = 1000 + Sa.*(1000./(1000-Sa)) + MmN2.*mn2_sw;  % [g per 1000g water content]

%------------------------------------------------------------------------
% Calculation of the solution volume V_sol per 1000 g water content 
%------------------------------------------------------------------------

V_sol = (1000 + Sa.*(1000./(1000-Sa)))./rho_sw + mn2_sw.* VN2; 
%[cm^3 per 1000 g water content]

%------------------------------------------------------------------------
% Calculation of the density of the N2-seawater-solution
%------------------------------------------------------------------------

rho= m_sol./V_sol; %[g/cm^3]
rho = rho .* 1000; % [kg/m^3]

end

