function [RHO] = density_swUnesco(press,temp,sal)

% DENSITY_SWUNESCO - calculates the density of seawater 
%     using the UNESCO formulae from 1983 and 1991
%
%     Usage: rho = density_swUnesco(p,T,S)
%
%     Input parameters (scalars or vectors): 
%         p        - pressure [MPa], 
%         T        - temperature [degC] 
%         S        - salinity [g/kg] 
%
%     Output parameters: 
%         rho      - density of seawater [kg/m^3]
%
%     References:
%     Fofonoff & Millard (1983) Unesco Technical Papers in Marine Science 44.
%     Poisson et al. (1991) Unesco Technical Papers in Marine Science 62.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       November 2009: typo removed from warning message (line 65)
%       September 2011: variable name changed: RSW -> RHO
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_SWUNESCO:Input','DENSITY_SWUNESCO: Missing input parameter! Please provide p, T and S!')
else

% Create 3-dimensional matrices for pressure-, temperature- and salinity-
% values with one element for each possible combination of specified P, T, S 
[PP,T,S] = ndgrid(press.*10,temp,sal); % pressure in bar

% Density calculation of seawater 
% Depending on salinity, chose corresponding equation
if ~any(sal > 42.)
   RHO = unesco83(PP,T,S);
else
   RHO = unesco91(PP,T,S);
%else
%   warning('Equations not valid for S>50!')
end

% Molarity and Molality of water in seawater
%M    = 18.0152;
%cH2O = RSW/M;
%mH2O = 1000/M * 1000./RSW;

end
%---------------------------------------------------
function [RSW]=unesco83(PP,T,S)
%     calculates the density of a "seawater" sample
%     using the UNESCO (1981,1983) formulae
%     (p=0-100 MPa,T=0-40 deg C, S=0.5-42 g/kg)
%


% Check if parameters are within the specified range for this equation
if ~any(S<0.5 | T<0 | T>40 | PP<0 | PP>1000)
else
   warning('DENSITY_SWUNESCO:OutOfParameterRange','pTS values out of range of UNESCO 1983!')
end


% Convert pressure to bars of sea pressure only
P = PP-1.013;
      
% RW: Density of pure reference water
A0 = 999.842594;
A1 = 6.793952e-02;
A2 = -9.095290e-03;
A3 = 1.001685e-04;
A4 = -1.120083e-06;
A5 = 6.536332e-09;

RW = ((((A5*T + A4).*T + A3).*T + A2).*T + A1).*T + A0;

% RTS: Density at T, S and P=1 atm
B0 = 8.24493e-01;
B1 = -4.0899e-03;
B2 = 7.6438e-05;
B3 = -8.2467e-07;
B4 = 5.3875e-09;
C0 = -5.72466e-03;
C1 = 1.0227e-04;
C2 = -1.6546e-06;
D0 = 4.8314e-04;

RTS = RW + ((((B4*T + B3).*T + B2).*T +B1).*T + B0).*S + ((C2*T + C1).*T + C0).*sqrt(S.*S.*S) + D0.*S.*S;

% RSW: Density at t, S and P
E0 = 19652.21;
E1 = 148.4206;
E2 = -2.327105;
E3 = 1.360477e-02;
E4 = -5.155288e-05;
KW = (((E4*T + E3).*T + E2).*T + E1).*T + E0;
H0 = 3.239908;
H1 = 1.43713e-03;
H2 = 1.16092e-04;
H3 = -5.77905e-07;
AW = ((H3*T + H2).*T + H1).*T + H0;
K0 = 8.50935e-05;
K1 = -6.12293e-06;
K2 = 5.2728D-08;
BW = (K2*T + K1).*T + K0;
M0 = -9.9348e-07;
M1 = 2.0816e-08;
M2 = 9.1697e-10;
B  = BW + ((M2*T + M1).*T + M0).*S;
I0 = 2.2838e-03;
I1 = -1.0981e-05;
I2 = -1.6078e-06;
J0 = 1.91075e-04;
A  = AW + ((I2*T + I1).*T + I0).*S + J0.*sqrt(S.*S.*S);
F0 = 54.6746;
F1 = -0.603459;
F2 = 1.09987e-02;
F3 = -6.1670e-05;
G0 = 7.944e-02;
G1 = 1.6483e-02;
G2 = -5.3009e-04;

KST0 = KW + (((F3*T + F2).*T + F1).*T + F0).*S + ((G2.*T + G1).*T + G0).*sqrt(S.*S.*S);
KSTP = KST0 + A.*P + B.*P.*P;

RSW = RTS;
dummy = find(P~=0.0);
RSW(dummy) =  RTS(dummy)./(1.0 - P(dummy)./KSTP(dummy));

%---------------------------------------------------
function [RSW]=unesco91(PP,T,S)
%     calculates the density of a "seawater" sample
%     using the UNESCO (1991) formulae
%     (p=1 atm, T=10-30 deg C, S=35-50 ppt)
%

% Check if parameters are within the specified range for this equation
if ~any(T<10 | T>30 | PP~=1.013)
else
   warning('DENSITY_SWUNESCO:OutOfParameterRange','pT values out of range of UNESCO 1991!')
end
 
% RW: Density of pure reference water
A0 = 999.842594;
A1 = 6.793952e-02;
A2 = -9.095290e-03;
A3 = 1.001685e-04;
A4 = -1.120083e-06;
A5 = 6.536332e-09;
RW = ((((A5*T + A4).*T + A3).*T + A2).*T + A1).*T + A0;

% RTS: Density at T, S and P=1 atm
B0 = 82.4427e-02;
B1 = -52.753e-04;
B2 = -51.17e-05;
B3 = 40.261e-06;
B4 = 11.5114e-05;
B5 = -14.791e-06;
B6 = 67.90e-08;
B7 = -15.886e-07;
B8 = -52.228e-08;
B9 = 20.750e-08;
RTS = RW + S.*(B0 + B1.*T + B2.*S + B3.*T.*T + B4.*T.*S + B5.*S.*S + B6.*T.*T.*T + B7.*T.*T.*S + B8.*T.*S.*S + B9.*S.*S.*S);

RSW = RTS;
