function D = diffcoeff_h2oco2(press,temp)

% DIFFCOEFF_H2OCO2 - calculates the diffusion coefficients of water
%      molecules dissolved in liquid CO2 
%
%      Usage: D = diffcoeff_h2oco2(P,T)
%
%      Input parameters (p,T: scalars or vectors):
%         p        - pressure [MPa]     (13.2 < P < 29.8)
%         T        - temperature [degC] (10 < T < 35)
%
%      Output parameter: 
%         D        - diffusion coefficient [m^2/s]
%
%      References: 
%      Xu et al. (2003) J. Phys. Chem. A 107, 1-3.
%
%      Comments: 
%      The empirical function for the diffusion coefficient was derived by
%      a fit of the data published by Xu et al. (2003).
%      It is based on the form [D = (a + bT)*mu_0/mu] with an addiditional
%      presure-dependent term. The parameter range specified above 
%      corresponds to the parameter range of the experimental data. 
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('DIFFCOEFF_H2OCO2:Input','DIFFCOEFF_H2OCO2: Missing input parameter! Please provide p and T!')
else
    [P, T] = ndgrid(press,temp);
    muo = visco_co2(0.1.*ones(size(press)),temp); % viscosity of liquid CO2 at standard pressure
    mu = visco_co2(press,temp); % viscosity of liquid CO2 at given pressure
    pscorr = muo./mu;
    
    a1 = 31.1135;
    a2 = 0.3988;
    a3 = 0.1586;
    a4 = 0.2982;
    
    %calculation of diffusion coefficient D
    D = (a1 + a2.*T ).*pscorr.*(a3 + a4./log(P)) ; %[10^-8 m^2/s]
    D = D.*1e-8; % [m^2/s]
    
end
