function [rho, cv, cp, s, u, h, f, JT] = eos_o2(press,temp)

% EOS_O2 -  calculates thermodynamic properties of O2
%     based on an equation of state
%
%     Usage: [rho cv cp s u h f JT] = eos_o2(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0 < p < 81.8) 
%         T       - temperature [degC] (-219 < T < 127)
%
%     Output parameters:
%         rho     - density [kg/m^3]
%         cv      - specific isochoric heat capacity [J/(kg K)]
%         cp      - specific isobaric heat capacity [J/(kg K)]
%         s       - specific entropy [J/(kg K)] 
%         u       - specific internal energy [J/kg] 
%         h       - specific enthalpy [J/kg]
%         f       - fugacity coefficient
%         JT      - Joule-Thomson coefficient [K/MPa]
%
%     All thermodynamic properties (except for pv) are calculated from the 
%     dimensionless Helmholtz energy Phi = A /(RT)
%     ( A(rho,T) = h_0(T)-R*T-T*s_0(rho,T) ) and its derivatives with 
%     respect to p and T.
%
%     Reference: 
%     Schmidt & Wagner (1985) Fluid Phase Equilibria 19, 175-200.
% 
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (February 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_O2:Input','EOS_O2: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<=0 | press>81.8) && ~any(temp< -219 | temp>127)
else
    bad = find(press<=0 | press > 81.8);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-219 | temp>127);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_O2:OutOfParameterRange','EOS_O2: Input values out of range!')
    warning('EOS_O2:OutOfParameterRange','Equations valid for p = 0-81.8 MPa (p>0), T=(-219) - 127 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T] = ndgrid(press*1d3,temp+273.15); % pressure in kPa, temperature in K

rhoc = 13.63; % critical density of O2 [mol/dm^3]
Tc = 154.581; % critical temperature of O2 [K]
T0 = 298.15; % reference temperature [K]
p0 = 1.01325; % refrence pressure [bar]
R = 8.31434; %  gas constant [J/(mol K)]
RO2 = 259.8;  % specific gas constant [J/kg/K]
tau = Tc./T; % inverse of reduced temperature
delta0 = p0/(R*T0*rhoc);
MMO2 = 15.999*2; % molar mass of O2

% define empty data structure for density 
rho= zeros(size(P,1),size(P,2),size(P,3)); 

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction
 
rhoguess = P./T./R; % starting value for solver from ideal gas law [mol/dm^3]

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(P(i,:));
            rho(i,:) = NaN;
        elseif isnan(T(:,j));
            rho(:,j) = NaN;
        else  
            EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            % use internal function to look up first guess for density 
            rho_init1 = rhoguess(i,j);
            rho_init = rho_init1; % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
                [rho(i,j),~,EXITFLAG] = fsolve(@(rho_O2) minfunc(rho_O2,P(i,j),T(i,j),rhoc,R,tau(i,j)), rho_init,options);
                % change initial value for fit
                rho_init = rho_init1 + counter*1*(-1)^(counter); 
                 if rho(i,j) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                 if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta = rho./rhoc; % reduced density

%------------------------------------------------------------------------
% calculation of ideal gas part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau,delta0);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau);

%------------------------------------------------------------------------
% calculation of thermodynamic properties
%------------------------------------------------------------------------

% density rho [kg/m^3]
rho = rho*MMO2; 

% Entropy [J/kgK]:
s = RO2.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir) ;

% Internal Energy [J/kg]:
u = RO2.*T.*tau.*(dphi0dtau + dphirdtau);

% Isochoric heat capacity [J/kgK]:
cv = -RO2.*tau.^2.*(dphi0dtautau + dphirdtautau);

% Enthalpy [J/kg]: 
h = RO2.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta.*dphirddelta);

% Isobaric heat capacity [J/kgK]:
cp = RO2.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta));

% Fugacity coefficient:
f = exp(phir + delta.*dphirddelta - log(1 + delta .*dphirddelta));

% Joule-Thomson coefficient [K/kPa]
JT = 1./RO2./rho .*(-(delta.*dphirddelta + delta.^2.*dphirddeltadelta + ...
    delta.*tau.*dphirddeltatau)./((1 + delta.*dphirddelta - ...
    delta.*tau.*dphirddeltatau).^2 - tau.^2.*(dphi0dtautau + ...
    dphirdtautau).*(1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)));

JT = JT .* 1d6 ; % convert to [K/MPa]

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau,delta0)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% k: coefficients of the correlation equation
k(1) = -0.740775d-3;
k(2) = -0.66493d-4;
k(3) =  0.250042d1;
k(4) = -0.214487d2;
k(5) =  0.101258d1;
k(6) = -0.944365;
k(7) =  0.145066d2;
k(8) =  0.749148d2;
k(9) =  0.414817d1;

% calculate phi and derivatives
phi0 = k(1)*tau.^1.5 + k(2)*tau.^(-2) + k(3)*log(tau) + k(4)*tau + ...
     k(5)*log(exp(k(7)*tau) -1) + k(6)*log(1 + 2/3*exp(-k(8)*tau)) + k(9) ...
    + log(delta./delta0);

dphi0dtau = 1.5*k(1)*tau.^0.5 - 2*k(2)*tau.^(-3) + k(3)./tau + k(4) + ...
    k(5)*(k(7)*exp(k(7)*tau))./(exp(k(7)*tau) -1) - ...
    k(6)*(2/3*k(8)*exp(-k(8)*tau))./(1 + 2/3*exp(-k(8)*tau));

dphi0dtautau = 0.75*k(1)*tau.^(-0.5) + 6*k(2)*tau.^(-4) - k(3)./tau.^2 - ...
    k(5)*(k(7).^2*exp(k(7)*tau))./(exp(k(7)*tau) -1).^2 - ...
    k(6)*(2/3*k(8).^2*exp(-k(8)*tau))./(1 + 2/3*exp(-k(8)*tau)).^2; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% calculation of sum terms with parameters ni,di,ti
%----------------------------------------------------------------------

ni = [0.3983768749
    -0.1846157457d1
     0.4183473197
     0.2370620711d-1
     0.9771730573d-1
     0.3017891294d-1
     0.2273353212d-1
     0.1357254086d-1
    -0.4052698943d-1
     0.5454628515d-3
     0.5113182277d-3
     0.2953466883d-6
    -0.8687645072d-4
    -0.2127082589
     0.8735941958d-1
     0.1275509190
    -0.9067701064d-1
    -0.3540084206d-1
    -0.3623278059d-1
     0.1327699290d-1
    -0.3254111865d-3
    -0.8313582932d-2
     0.2124570559d-2
    -0.8325206232d-3
    -0.2626173276d-4
     0.2599581482d-2
     0.9984649663d-2
     0.2199923153d-2
    -0.2591350486d-1
    -0.1259630848
     0.1478355637
    -0.1011251078d-1];
  
ed2 = exp(-delta.^2);
ed4 = exp(-delta.^4);

di = [1 1 1 2 2 2 3 3 3 6 7 7 8 1 1 2 2 3 3 5 6 7 8 10 2 3 3 4 4 5 5 5];
ti = [0 1.5 2.5 -0.5 1.5 2 0 1 2.5 0 2 5 2 5 6 3.5 5.5 3 7 6 8.5 4 6.5 5.5 ...
    22 11 18 11 23 17 18 23];

%----------------------------------------------------------------------
% calculation of function output values
%----------------------------------------------------------------------

% create empty matrices
phir =  zeros(size(tau,1),size(tau,2)); %  phi_r: residual of Helmholtz energy
dphirddelta =  zeros(size(tau,1),size(tau,2)); % d(phi_r)/d(delta)
dphirddeltadelta =  zeros(size(tau,1),size(tau,2)); % d^2(phi_r)/d(delta^2)
dphirdtau = zeros(size(tau,1),size(tau,2));         % d(phi_r)/d(tau)
dphirdtautau = zeros(size(tau,1),size(tau,2));  % d^2(phi_r)/d(tau^2)
dphirddeltatau = zeros(size(tau,1),size(tau,2)); % d^2(phi_r)/d(delta)d(tau)

for i = 1:13
    phir = phir + ni(i).*delta.^(di(i)).*tau.^(ti(i));
    
    dphirddelta = dphirddelta + ni(i).*di(i).*delta.^(di(i)-1).*tau.^(ti(i));
    
    dphirddeltadelta = dphirddeltadelta + ni(i).*di(i).*(di(i)-1)...
         .*delta.^(di(i)-2).*tau.^(ti(i));
     
    dphirdtau = dphirdtau + ni(i).*ti(i).*delta.^(di(i)).*tau.^(ti(i)-1);
    
    dphirdtautau = dphirdtautau + ni(i).*ti(i).*(ti(i)-1)...
        .*delta.^(di(i)).*tau.^(ti(i)-2);
    
    dphirddeltatau = dphirddeltatau + ni(i).*di(i).*ti(i) ...
        .*delta.^(di(i)-1).*tau.^(ti(i)-1);
end 

for i = 14:24
    phir = phir + ed2.*ni(i).*delta.^(di(i)).*tau.^(ti(i));
    
    dphirddelta = dphirddelta + ed2.*ni(i).*tau.^(ti(i)).* ...
        (di(i).*delta.^(di(i)-1) - 2.*delta.^(di(i)+1));
    
        dphirddeltadelta = dphirddeltadelta + ed2.*ni(i).*tau.^(ti(i)).* ...
        (di(i).*(di(i)-1).*delta.^(di(i)-2) + 4.*delta.^(di(i)+2)...
        - 2.*(2.*di(i) + 1).*delta.^di(i));
    
    dphirdtau = dphirdtau + ed2.*ni(i).*ti(i).*delta.^(di(i)).*tau.^(ti(i)-1);
    
    dphirdtautau = dphirdtautau + ed2.*ni(i).*ti(i).*(ti(i)-1) ...
        .*delta.^(di(i)).*tau.^(ti(i)-2);
    
    dphirddeltatau = dphirddeltatau + ed2.*ni(i).*ti(i).*tau.^(ti(i)-1).* ...
        (di(i).*delta.^(di(i)-1) - 2.*delta.^(di(i)+1));
end 

for i = 25:32
    phir = phir + ed4.*ni(i).*delta.^(di(i)).*tau.^(ti(i));
    
     dphirddelta = dphirddelta + ed4.*ni(i).*tau.^(ti(i)).* ...
        (di(i).*delta.^(di(i)-1) - 4.*delta.^(di(i)+3));
    
     dphirddeltadelta = dphirddeltadelta + ed4.*ni(i).*tau.^(ti(i)).* ...
        (di(i).*(di(i)-1).*delta.^(di(i)-2) + 16.*delta.^(di(i)+6)...
        - 4.*(2.*di(i) + 3).*delta.^(di(i)+2));
    
     dphirdtau = dphirdtau + ed4.*ni(i).*ti(i).*delta.^(di(i)).*tau.^(ti(i)-1);
        
     dphirdtautau = dphirdtautau + ed4.*ni(i).*ti(i).*(ti(i)-1) ...
        .*delta.^(di(i)).*tau.^(ti(i)-2);
    
     dphirddeltatau = dphirddeltatau + ed4.*ni(i).*ti(i).*tau.^(ti(i)-1).* ...
        (di(i).*delta.^(di(i)-1) - 4.*delta.^(di(i)+3));
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc(rho,P,T,rhoc,R,tau)
% minimization function to calculate density at given pressure
% based on the equation p = rho*R*T*(1 + rho/rohc * d(phir)/d(rho/rhoc)) 

delta = rho/rhoc;

% calculate d(phir)/d(rho/rhoc)
[~, ~, ~, ~, dphirddelta] = helmholtz_residual(delta,tau);

% calculate density
RHOeos = P /(R*T*(1+delta*dphirddelta));

% calculate difference between initial rho and "new" calculated rho
f = rho - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
