function [f_gh] = fugacity_co2gh(temp,N_CO2)

% FUGACITY_CO2GH - calculates the fugacity of CO2 in CO2-hydrate
%
%       Usage: f = fugacity_co2gh(T,N)
%
%       Input parameters (scalars or vectors): 
%           T      - temperature [degC] 
%           N      - moles of CO2 per mole of water in hydrate phase (0 < N < 4/23)
%
%       Output parameter:
%           f      - fugacity of CO2 in CO2_hydrate [MPa]
%
%       References:
%       Cole & Goodwin (1990) Chem. Eng. Sci. 45, 569-573.
%       Klauda $ Sandler (2003) Chem. Eng. Sci. 58, 27-41 (Langmuir constants)
%
%       Comments: 
%       This algorithm is entirely based on a theoretical equation 
%       - no validation by experimental data conducted.
%
%       An approximate N can be derived from the output of the
%       toolbox function fugacity_h2och: 
%       N = 1/23 * theta_s + 3/23 * theta_l
%
%       At hydrate dissociation pressure, the match between gas phase 
%       fugacity of CO2 and hydrate phase fugacity of CO2 is not satisfying.
%       N (as calculated from the output of fugacity_h2och) is decreased
%       by about 2%.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%     Modification history: 
%       November 2009: indices set in correct order (lines 63-64)
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       November 2012: syntax changed to Matlab2012 style
%       December 2012: error in indexing during loop corrected
%       March 2013:     function was completely rewritten (different set of
%                       equations)
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('FUGACITY_CO2GH:Input','FUGACITY_CO2GH: Missing input parameter! Please provide T, and N!')
else

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[T, N] = ndgrid(temp+273.15, N_CO2); %  T in K

nu_s = 1/23; % number of small cavities
nu_l = 3/23; % number of large cavities


%------------------------------------------------------------------------
% Calculation of the Langmuir constants (s = small cage, l = large cage)
%------------------------------------------------------------------------

% As = 25.354129;
% Al = 23.632139;
% Bs = 3277.0691;
% Bl = 3697.1701;
% % Langmuir constants: Duan
% C_CO2_s = exp(-As + Bs./T);
% C_CO2_l = exp(-Al + Bl./T);

Ac_CO2_s = -24.9824;
Ac_CO2_l = -22.4037;
Bc_CO2_s = 2743.7375;
Bc_CO2_l = 3171.7604;
Dc_CO2_s = 31948.6496;
Dc_CO2_l = 0;

% Langmuir constants: Klauda
C_CO2_s = exp(Ac_CO2_s + Bc_CO2_s./T + Dc_CO2_s./T.^2);
C_CO2_l = exp(Ac_CO2_l + Bc_CO2_l./T + Dc_CO2_l./T.^2);
%------------------------------------------------------------------------
% Calculation of CO2 fugacity in CO2-hydrate
%------------------------------------------------------------------------

% solve quadratic equation
r1 = C_CO2_s .* C_CO2_l .* (N - nu_s - nu_l);
r2 = N.*C_CO2_s + N.*C_CO2_l - C_CO2_s.*nu_s - C_CO2_l.*nu_l;

p1 = r2./r1;
p2 = N./r1;

f_gh = - p1/2 + sqrt(p1.^2/4 - p2);

f_gh = f_gh.*1d-6; % fugacity in [MPa]
end