function [f_co2, Theta_CO2_s, Theta_CO2_l] = fugacity_h2och(press,temp)

% FUGACITY_H2OCH -  calculates the fugacity of water in CO2 SI-hydrate 
%
%       Usage: [f theta_s, theta_l] = fugacity_h2och(p,T)
%
%       Input parameters (scalars or vectors): 
%           p         - pressure [MPa]  
%           T         - temperature [degC] (-133 < T < 57)
%   
%       Output parameter:
%           f         - fugacity of water in CO2 SI-hydrate [MPa]
%           theta_s   - fractional occupancy of small hydrate cages
%           theta_l   - fractional occupancy of large hydrate cages
%           
%       References:
%       Klauda & Sandler (2003) Chem. Eng. Sci. 58, 27-41.
%       Klauda & Sandler (2000) Ind. Eng. Chem. Res. 39, 3377-3386.
%
%       Comments: 
%       Coefficients A-D for the vapor pressure of the empty hydrate
%       lattice are derived from a fit to match the liquid water fugacity
%       as provided by the toolbox function fugacity_sw to the water
%       fugacity as calculated by the Klauda & Sandler formalism at
%       the hydrate phase boundary (calculated by phase_co2sw).
%
%       Fugacities of CO2 in the hydrate phase are approximated as
%       fugacities of gas phase CO2. For the calculation of hydrate phase
%       fugacities, the total mount of CO2 in the hydrate phase has to be
%       known.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       December 2009: Parameter name in header corrected (line 12)
%       November 2012: syntax changed to Matlab2012 style
%                       error corrected: fugacity coefficient is multiplied
%                       with pressure to obtain fugacity
%       March 2013:     Fractional cage occupancies added as output
%                       parameters
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('FUGACITY_H2OCH:Input','FUGACITY_H2OCH: Missing input parameter! Please provide p and T!')
else


%------------------------------------------------------------------------
% Check validity of data range!
%------------------------------------------------------------------------

if ~any(temp<-133 | temp>57) 
else
    bad = find(temp<-133 | temp > 57);
    temp(bad)=NaN*ones(size(bad));
    warning('FUGACITY_H2OCH:OutOfParameterRange','FUGACITY_H2OCH: Input values out of range!')
    warning('FUGACITY_H2OCH:OutOfParameterRange','Equations valid for T = -133-57 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P, T] = ndgrid(press,temp+273.15); %  pressure in MPa, temperature in K

% call external function to calculate gas phase fugacity coefficient phi_CO2
[~ , ~ , ~, ~, ~, ~, phi_CO2] = eos_co2(press,temp); 
f_CO2 = phi_CO2.*1e6.*P; % fugacity in [Pa]

Na = 6.0222045e23; % Avogadro's number
Nb = 46; % water molecules per unit cell
R = 8.314472; % gas constant [J/mol/K]

%------------------------------------------------------------------------
% Calculation of the molar volume of empty SI hydrate
% Comment: Power of 3 inserted into equation (seems to be missing in paper)
%------------------------------------------------------------------------

V_b = (11.835 + 2.217e-5.*T + 2.242e-6.*T.^2).^3.*1e-30.*Na./Nb - ...
    8.006e-9.*P + 5.448e-12.*P.^2; %  [m3/mol]

%------------------------------------------------------------------------
% Calculation of the Langmuir constants (s = small cage, l = large cage)
%------------------------------------------------------------------------

% As = 25.354129;
% Al = 23.632139;
% Bs = 3277.0691;
% Bl = 3697.1701;
% % Langmuir constants: Duan
% C_CO2_s = exp(-As + Bs./T);
% C_CO2_l = exp(-Al + Bl./T);

Ac_CO2_s = -24.9824;
Ac_CO2_l = -22.4037;
Bc_CO2_s = 2743.7375;
Bc_CO2_l = 3171.7604;
Dc_CO2_s = 31948.6496;
Dc_CO2_l = 0;

% Langmuir constants: Klauda
C_CO2_s = exp(Ac_CO2_s + Bc_CO2_s./T + Dc_CO2_s./T.^2);
C_CO2_l = exp(Ac_CO2_l + Bc_CO2_l./T + Dc_CO2_l./T.^2);
%------------------------------------------------------------------------
% Calculation of the cage occupancies
%------------------------------------------------------------------------

Theta_CO2_s = C_CO2_s .*f_CO2./(1 + C_CO2_s .*f_CO2);
Theta_CO2_l = C_CO2_l .*f_CO2./(1 + C_CO2_l .*f_CO2);

%------------------------------------------------------------------------
% Calculation of chemical potential difference
%------------------------------------------------------------------------

Delta_mu_CO2 = -R.*T.*(2./46.*log(1 - Theta_CO2_s) + ...
    6./46.*log(1 - Theta_CO2_l));

%------------------------------------------------------------------------
% Calculation of the vapor pressure of the empty hydrate lattice
%------------------------------------------------------------------------

Ai_CO2 =   86.048008459101823;
Bi_CO2 =   -69.918008082202732;
Ci =  -4.121018699895194e+02;
Di_CO2 =  -0.232652658809132;

% Ai_CO2 =   66.253663044461746;
% Bi_CO2 =   -1.280651307655475e+02;
% Ci =  -3.199441252175932e+02;
% Di_CO2 =  -0.162283883758242;

P_sat_CO2 = exp(Ai_CO2.*log(T) + Bi_CO2./T + Ci + Di_CO2.*T); % [Pa]

%------------------------------------------------------------------------
% Calculation of the fugacity of the empty hydrate lattice
%------------------------------------------------------------------------

f_wb_CO2 = P_sat_CO2.*exp(V_b.*(P.*1e6 - P_sat_CO2)./R./T); % [Pa]

%------------------------------------------------------------------------
% Calculation of the fugacity of water in CO2 gas hydrate 
%------------------------------------------------------------------------

f_co2 = f_wb_CO2.*exp(- Delta_mu_CO2./R./T) .*1e-6; % [MPa]


end
