function [fH2S] = fugacity_h2ssw(press,temp,sal,mh2s)

% FUGACITY_H2SSW -  calculates the fugacity of dissolved H2S gas
%     in equilibrium with seawater
%
%     Usage: f = fugacity_h2ssw(p,T,S,m)
%
%     Input parameters (scalars or vectors, m: scalar/matrix of size pxtxS): 
%           p      -  pressure [MPa]      (0.1 < p < 20)
%           T      -  temperature [degC]  (0 < T < 127)
%           S      -  practical salinity  (0 < S < 350)
%           m      -  molality of dissolved H2S in seawater [mol/kg],
%                       default is saturation concentration
%
%     Output parameter: 
%           f      - fugacity of H2S in seawater [MPa]
%
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 2779-2795. (partial molar volume of H2S)
%     Duan et al (2007) Energy & Fuels 21, 2056-2065. (gamma term)
%
%     Comments: 
%     Formalism is not valid for pT conditions where pure H2S is liquid.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_H2SSW:Input','SOLUHENRY_H2SSW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>120)
else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_H2SSW:OutOfParameterRange','SOLUHENRY_H2SSW: Input values out of range!')
    warning('SOLUHENRY_H2SSW:OutOfParameterRange','Equations valid for p = 0.1-35 MPa, T= 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

   if nargin == 3 % if no H2S concentration is provided by user, assume saturation
       %call external function to calculate solubility
        mh2s = solu_h2ssw(press,temp,sal);
   end
 

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
% call external function to calculate pressure and salinity corrected Henry
% constant
[~,~,kH] = soluhenry_h2ssw(press,temp,sal,0.1); % [1/MPa]
x = mh2s./(mh2s + 55.5); % mole fraction of O2

fH2S = x./kH; % fugacity
end
% ------------------------------------------------------------
