function [keq]= kequilib_sw(press,temp,sal,species)

% KEQUILIB_SW - calculates the stoichiometric equilibrium constants
%      of various equilibrium reactions in seawater
%      The constants are calculated with respect to seawater pH scale.
%      The constants assume concentrations in molality.
%
%      Usage:  keq = kequilib_sw(P,T,S,'species')
%
%      Input parameters (scalars or vectors): 
%           p        -  pressure [MPa] (0.1-100 MPa), 
%           T        -  temperature [degC] (0-45 C) 
%           S        -  salinity [g/kg] (0-45 g/kg)
%          'species' -  choice of the chemical substance for
%                       which the equilibrium constant is calculated:
%   1           'h2o'       H2O <--> H+ + OH-
%   2           'co2'       CO2 + H2O <--> H+ + HCO3-
%   3           'hco3'      HCO3- <--> H+ + CO32-
%   4           'boh4'      [B(OH)4]- <--> H+ + B(OH)3
%   5           'h3po4'     H3PO4 <--> H+ + H2PO4-
%   6           'h2po4'     H2PO4- <--> H+ + HPO42-
%   7           'hpo4'      HPO42- <--> H+ + PO43-
%   8           'hno3'      HNO3 <--> H+ + NO3-
%   9           'nh4'       NH4+ <--> H+ + NH3
%   10          'h2s'       H2S <--> H+ + HS-
%   11          'arag'      aragonite: CaCO3 <--> Ca2+ + CO32-
%   12          'calc'      calcite: CaCO3 <--> Ca2+ + CO32-
%   13          'mnco3'     MnCO3 <--> Mn2+ + CO32-
%   14          'feco3'     FeCO3 <--> Fe2+ + CO32-
%   15          'baso4'     barite: BaSO4 <--> Ba2+ + SO42-
%   16          'srso4'     celestite: SrSO4 <--> Sr2+ + SO42-
%   17          'so4'       HSO4- <--> H+ + SO4-- 
%   18          'hf'        HF <--> H+ + F-
%   19          'sio2'      H4SiO4 <--> H+ + H3SiO4-
%
%      Output parameter: 
%           keq      - equilibrium constant [mol/kg(H2O)] or [(mol/kg(H2O))^2]
%
%   References:
%   Boudreau (1996) Computers & Geosciences 22, 479-496.
%   Clegg & Whitfield (1995) GCA 59(12), 2403-2421.
%   Fofonoff & Millard (1983) UNESCO Technical Paper in Marine Science 44.
%   Luff et al. (2001) Computers & Geosciences 27, 157-169.
%   Millero (1983) In: Ripley & Chester (eds.) Chemical Oceanography Vol.8,
%      Academic Press, p.1-88.
%   Millero (1995) GCA 59(4), 661-677.
%   Millero et al. (2006) Mar. Chem. 100, 80-94.
%   Millero (2007) Chem. Rev. 107, 308-341.
%   Stumm & Morgan (1996) Aquatic Chemistry, Wiley&Sons.
%   Van Cappellen & Wang (1996) Amer. J. Sci. 296, 197-243.
%   Zeebe & Wolf-Gladrow (2001) CO2 in seawater: Equilibrium, kinetics
%     isotopes, Elsevier.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
%     Modification history: 
%       February 2013: Output converted from mol/kg(solution) to
%       mol/kg(H2O), additional species implemented
%------------------------------------------------------------------------



%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('KEQUILIB_SW:Input','KEQUILIB_SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp< 0 | temp>45) ...
        && ~any( sal<0 | sal>45)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>45);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>45);
    sal(bad)=NaN*ones(size(bad));

    warning('KEQUILIB_SW:OutOfParameterRange','KEQUILIB_SW: Input values out of range!')
    warning('KEQUILIB_SW:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=0-45 degC, S=0-45.')
end

% set default case
if nargin == 3
   species = '';
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*10,temp+273.15,sal); %pressure in bar, temperature in K
tt = T - 273.15;

%------------------------------------------------------------------------
% Calculation of equilibrium constant for selected species
%------------------------------------------------------------------------


switch species
    case 'h2o'
        % Millero (1995)
        % Note: Dickson uses 148.9652 (subtracts 0.015 from the first constant)
        lnk0 = 148.9802 - 13847.26./T - 23.6521*log(T) ...
            + S.^.5 .* (-5.977 + 118.67./T + 1.0495*log(T)) ...
            - 0.01615*S;

        dV = -25.60 + 0.2324*tt - 3.6246D-3*tt.^2;

        dcomp = -5.13 + 0.0794*tt;
        corr = log(1- 0.001005.*S).^2; % correction factor for converting (mol/kg(solution))^2 to (mol/kg(H2O))^2

%        dV = -20.02 + 0.1119*tt - 1.4096D-3*tt.^2;

    case 'co2'
        % Millero (2006): T=0-50degC, S=0-50 
        pkc01 = -126.34048 + 6320.813./T + 19.568225*log(T);

        a = 13.4191*S.^.5 + 0.0331*S - 5.33D-5*S.^2;
        b = -530.123*S.^.5 - 6.103*S;
        c = -2.06950*S.^.5;

        pkc1 = pkc01 + a + b./T + c.*log(T);
        lnk0 = log(10.^-pkc1);

        dV = -25.50 + 0.1271*tt;
        dcomp = -3.08 + 0.0877*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

        % Millero (1995)
%        lnk0 = 2.18867 - 2275.0360./T - 1.468591*log(T) ...
%              + S.^.5 .* (-0.138681 - 9.33291./T) ...
%              + 0.0726483*S - 0.00574938*S.^1.5;
%        dV = -25.50 + 0.151*(S-34.8) - 0.1271*tt;
%        dcomp = -(3.08 + 0.578*(S-34.8) - 0.0877*tt);

    case 'hco3'
        % Millero (2006): T=0-50degC, S=0-50 
        pkc02 = -90.18333 + 5143.692./T + 14.613358*log(T);

        a = 21.0894*S.^.5 + 0.1248*S - 3.687e-4*S.^2;
        b = -772.483*S.^.5 - 20.051*S;
        c = -3.3336*S.^.5;

        pkc2 = pkc02 + a + b./T + c.*log(T);
        lnk0 = log(10.^-pkc2);

        dV = -15.82 - 0.0219*tt;
        dcomp = 1.13 - 0.1475*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

        % Millero (1995)
%        lnk0 = -0.84226 - 3741.1288./T - 1.437139.*log(T) ...
%              + S.^.5 .* (-0.128417 - 24.41239./T) ...
%              + 0.1195308*S - 0.0091284*S.^1.5;
%        dV = -15.82 - 0.321*(S-34.8) + 0.0219*tt);
%        dcomp = -(-1.13 + 0.314*(S-34.8) + 0.1475*tt);

    case 'boh4'
        lnk0 = (-8966.90 -2890.51*S.^.5 - 77.942*S ...
            + 1.726*S.^1.5 - 0.0993*S.^2) ./ T ...
            + 148.0248 + 137.194*S.^.5 + 1.62247*S ...
            + (-24.4344 - 25.085*S.^.5 - 0.2474*S) .* log(T) ...
            + 0.053105 * S.^.5 .* T;

        dV = - 29.48 - 0.1622*tt + 2.608D-3*tt.^2;
        dcomp = -2.84;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

%        dV = -29.48 - 0.295*(S-34.8) - 0.1622*tt + 2.608D-3*tt.^2;
%        dcomp = - (2.84 - 0.354*(S-34.8));

    case 'h3po4'
        lnk0 = 115.54 - 4576.752./T - 18.453*log(T) ...
               + S.^.5 .* (0.69171 - 106.736./T) ...
               + S .* (-0.01844 - 0.65643./T);

        dV = -14.51 + 0.1211*tt - 0.321D-3*tt.^2;
        dcomp = -2.67 + 0.0427*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

    case 'h2po4'
        lnk0 = 172.1033 - 8814.715./T - 27.927*log(T) ...
                + S.^.5 .* (1.3566 - 160.34./T) ...
                + S .* (-0.05778 + 0.37335./T);

        dV = -23.12 + 0.1758*tt - 2.647d-3*tt.^2;
        dcomp = -5.15 + 0.0900*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

    case 'hpo4'
        lnk0 = -18.126 - 3070.75./T ...
            + S.^.5 .* (2.81197 + 17.27039./T) ...
             + S .* (-0.09984 - 44.99486./T);

        dV = -26.57 + 0.2020*tt - 3.042D-03*tt.^2;
        dcomp = -4.08 + 0.0714*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

    case 'hno3'
        lnk0 = log(23.44) * ones(size(T));

        dV = 0;
        dcomp = 0;
        corr = 0;

    case 'nh4'
        lnk0 = -6285.33./T + 1.635D-4*T - 0.25444 ...
            + S.^.5 .* (0.46532 - 123.7184./T) ...
            + S .* (-0.01992 + 3.17556./T);

% Note: Millero (1983) gives dV and dcomp for the reaction
%       NH3 + H2O = NH4+ + OH-
%   whereas the value calculated here is for the reaction
%       NH4+ = NH3 + H+
%   In order to get the desired value, dV and dcomp 
%   for water are subtracted from the given values 
%   and the negative of this resulting number is used.

        dVn = -26.43 + 0.0889*tt - 0.905D-3*tt.^2;
        dcompn = -5.03 + 0.0814*tt;
        dVw = -25.60 + 0.2324*tt - 3.6246D-3*tt.^2;
        dcompw = -5.13 + 0.0794*tt;

%        dV = -20.02 + 0.1119*tt - 1.4096D-3*tt.^2;
        dV = - (dVn - dVw);
        dcomp = - (dcompn - dcompw);
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

    case 'h2s'
        lnk0 = 225.838 - 13275.3./T - 34.6435*log(T) ...
            + 0.3449*S.^.5 - 0.0274*S;

        dV = -14.80 + 0.0020*tt - 0.400D-3*tt.^2;
        dcomp = 2.89 + 0.054*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)

%        dV = -11.07 + 0.009*t - 0.942-03*t*t;

    case 'arag'
        logkarag = -171.945 - 0.077993*T + 2903.293./T ...
             + 71.595*log10(T) + S.^.5 .* (-0.068393 ...
            + 1.7276D-3*T + 88.135./T) ...
            - 0.10018*S + 5.9415D-3*S.^1.5;

        lnk0 = log(10.^logkarag);

        dV = -48.76 - 0.5304*tt;
        dcomp = -11.76 + 0.3692*tt;
        corr = log(1- 0.001005.*S).^2; % correction factor for converting (mol/kg(solution))^2 to (mol/kg(H2O))^2

%        dV = -(48.76 - 0.5304*tt) + 2.8;

    case 'calc'
        logkcalc = -171.9065 - 0.077993*T + 2839.319./T ...
            + 71.595*log10(T) + S.^.5 .* (-0.77712 ...
            + 2.8426D-3*T + 178.34./T) ...
            - 0.07711*S + 4.1249D-3.*S.^1.5;

        lnk0 = log(10.^logkcalc);
        dV = -48.76 - 0.5304*tt;
        dcomp = -11.76 + 0.3692*tt;
        corr = log(1- 0.001005.*S).^2; % correction factor for converting (mol/kg(solution))^2 to (mol/kg(H2O))^2

%        dV = -(48.76 - 0.5304*tt);

    case 'mnco3'    % (10^-8.5 = 3.1623D-09)
        lnk0 = log(10^-8.5) * ones(size(T));
        dV = 0;
        dcomp = 0;
        corr = 0;

    case 'feco3'    % (10^-8.4=3.9811e-09)
        lnk0 = log(10^-8.4) * ones(size(T));
        dV = 0;
        dcomp = 0;
        corr = 0;

    case 'baso4'    % Stumm & Morgan, p.983
        logkbaso4 = 136.035 - 7680.41./T - 48.595*log10(T);
        lnk0 = log(10.^logkbaso4);
        dV = 0;
        dcomp = 0;
        corr = 0;

    case 'srso4'    % Stumm & Morgan, p.983
        logksrso4 = -14805.9622-2.4660924*T + 756968.533./T ...
             - 40553604./T.^2 + 5436.3588*log10(T);

        lnk0 = log(10.^logksrso4);
        dV = 0;
        dcomp = 0;
        corr = 0;
        
     case 'so4'
        % Zeebe & Wolf-Gladrow, pp. 260
        
        I = 19.924 .* S ./ (1000 - 1.005 .* S); % ionic strength
   
        lnk0 = - 4276.1./T + 141.328 - 23.093 .* log(T) + ...
            (- 13856./T + 324.57 - 47.986 .* log(T)) .* I.^(0.5) + ...
            (35474./T - 771.54 + 114.723 .* log(T)) .* I - ...
            2698./T.* I.^(1.5) + 1776./T.*I.^2 + log(1 - 0.001005 .* S);
        
        dV = -18.03 + 0.0466.*tt + 0.316e-3.*tt.^2;
        dcomp = -4.53 + 0.09.*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)
        
     case 'hf'
        % Zeebe & Wolf-Gladrow, pp. 260
        
        I = 19.924 .* S ./ (1000 - 1.005 .* S); % ionic strength
        
        % equilibrium constant for sulfate
        lnk0_s = - 4276.1./T + 141.328 - 23.093 .* log(T) + ...
            (- 13856./T + 324.57 - 47.986 .* log(T)) .* I.^(0.5) + ...
            (35474./T - 771.54 + 114.723 .* log(T)) .* I - ...
            2698./T.* I.^(1.5) + 1776./T.*I.^2 + log(1 - 0.001005 .* S);
        
        St = 0.02824.*S./35 ; % total sulfate concentration
        
        lnk0 = 1590.2./T - 12.641 + 1.525 .* I.^(0.5) ...
               + log(1 - 0.001005 .* S) + log(1 + St./exp(lnk0_s));
        
        dV = -9.78 + -0.009.*tt + -0.942e-3.*tt.^2;
        dcomp = -3.91 + 0.054.*tt;
        corr = log(1- 0.001005.*S); % correction factor for converting mol/kg(solution) to mol/kg(H2O)
        

         case 'sio2'
        % Millero (2007)
        
        I = 19.924 .* S ./ (1000 - 1.005 .* S); % ionic strength
   
        lnk0 = 117.40 - 8904.2./T - 19.334.*log(T) + ...
            (3.5913 - 458.79./T) .* I.^(0.5) + ...
            (-1.5998 + 188.74./T) .* I + (0.07871 - 12.1652./T).* I.^2;

        % dV and dcomp are estimated from boric acid (s. Millero, 2007)
        dV = - 29.48 - 0.1622*tt + 2.608D-3*tt.^2;
        dcomp = -2.84;
        corr = 0; 
        
    otherwise
        lnk0 = log(10^-14) * ones(size(T));
        dV = 0;
        dcomp = 0;
        corr = 0;
end

%------------------------------------------------------------------------
% Calculation of pressure correction
%------------------------------------------------------------------------


% pressure correction is using the relation:
%  ln(Kp/K0) = - dV/(R*T)*PP + dcomp/(2*R*T)*PP^2
% where dV = partial molar volume change
%       dcomp = change in compressibility
%       PP = water pressure over atmospheric pressure [bar]
% Valid for pressures up to 1000 bar.
%

R = 83.131; % gas constant [mol*bar/K]

%PP = P - 1.0;
%lnkp = lnk0 - dV./(R*T).*P + dcomp./(2*R*T).*P.^2;
% dcomp/1000 because otherwise pressure effect is too large
% and does not fit to pH,DIC,CA data of the Atlantic and 
% Pacific ocean (see Millero, 2007). The same has been
% done in C.CANDI (Boudreau, 1996; Luff et al., 2001).

lnkp = lnk0 - dV./(R*T).*P + dcomp/1000./(2*R*T).*P.^2;
keq = exp(lnkp - corr); % equilibrium constant in [mol/kg] or [mol^2/kg^2]

%------------------------------------------------------------------------
% conversion from mol/kg -> mmol/l and (mol/kg)^2 -> (mmol/l)^2
%------------------------------------------------------------------------
%
%switch species
%    case {'h2o','arag','calc','mnco3','feco3','baso4','srso4'}
%        keq = keq2 * 1.D6 .* density_sw(press,temp,sal).^2;
%    otherwise
%        keq = keq2 * 1.D3 .* density_sw(press,temp,sal);
%end
%
end
