function ph2s = mol2pp_h2s(press,temp,sal,mh2s)

% MOL2PP_H2S - calculates the partial pressure of H2S for a given H2S
%      concentration in aqueous solution using Henry's law.
%      The Henry constant for H2S gas in water is calculated as a function
%      of pressure, temperature and salinity from a multiparametric equation.
%
%      Usage: pp = mol2pp_h2s(p,T,S,m)
%
%      Input parameters (p,T,S: scalars or vectors; m: scalar/matrix of size P x T x S): 
%           p        -  pressure [MPa]      (0.1 < p < 20)
%           T        -  temperature [degC]  (0 < T < 127)
%           S        -  practical salinity  (0 < S < 350)
%           m        -  molal concentration of H2S [mol/kg]
%
%      Output parameter: 
%           pp       -  partial pressure of H2S [MPa] 
% 
%     References:
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 2779-2795 (partial molar volume of H2S)
%     Duan et al. (2007) Energy & Fuels 21, 2056-2065. (gamma term)
%        
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('MOL2PP_H2S:Input','MOL2PP_H2S: Missing input parameter! Please provide P,T,S and m!')
else
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal> 120) 
else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('MOL2PP_H2S:OutOfParameterRange','MOL2PP_H2S: Input values out of range!')
    warning('MOL2PP_H2S:OutOfParameterRange','Equations valid for P = 0.1 - 35 MPa, T = 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P] = ndgrid(press,temp,sal); 

% calculate mole fraction x of dissolved H2S
x = mh2s./(55.5 + mh2s);

% call external function to calculate Henry constant KH

[~,~, KH] = soluhenry_h2ssw(press,temp,sal,0.1); % [1/MPa]

F = x./KH; % calculate fugacity

% solve EOS numerically for ph2s at given fugacity
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

ph2s=zeros(size(P)); % empty matrix for partial pressure

for i = 1:length(press)
    for j = 1:length(temp)
        for k=1:length(sal)
         if isnan(press(i)) || isnan(temp(j)) || isnan(sal(k)) || isnan(F(i,j,k))
            ph2s(i,j,:) = NaN;
         else
            % find starting value for P_h2s
            [~,ff] = eos_h2s(press(i),temp(j)); % f = fugacity coefficient
            p_init = press(i).*(F(i,j,k)./(press(i).*ff));
            [PP_h2s] = fsolve(@(P_h2s) minfunc(P_h2s,press(i),temp(j),F(i,j,k)),p_init,options);
            ph2s(i,j,k) = PP_h2s;
         end
         end
    end
end

 
end

% ----------------------------------------------------------

function D = minfunc(ph2s,p,temp,F)
% minimization function to calculate the partial pressure of H2S in the
% gas phase

% calculate fugacity coefficient
[~,f] = eos_h2s(p,temp); % f = fugacity coefficient

% calculate difference between initial fugacity coefficient and new
% fugacity coefficient
D = f - F./ph2s;
