function [eqsys] = ph_analyt(press,temp,sal,TA,DIC,TB,TS,TN,TP,TSi,TSO4,TF)

% PH_ANALYT -  calculates the pH of seawater and the equilibrium 
%      concentrations of the species H+, CO2, HCO3-, CO32-, B(OH)4-,
%      B(OH)3, H2S, and HS- based on their total concentrations
%      (DIC, TBOH4, TH2S) and the total alkalinity TALK.
%
%      The concentrations of dissolved ammonium, phosphate, silicate, 
%      sulfate, and fluoride species are only calculated from this pH.
%      To assess their influence on the pH compare with the numerical 
%      result from the routine PH_MODEL.
%
%      Usage:  [eqsys] = ph_analyt(p,T,S,TA,DIC,TBOH4,TH2S,TNH4,TPO4,TSi,TSO4,TF)
%
%      Input parameters (scalars only):
%           p      -  pressure [MPa], 
%           T      -  temperature [degC] 
%           S      -  salinity [g/kg]
%           TALK   -  total alkalinity [eq/kg]
%           DIC    -  total dissolved inorganic carbon [mol/kg]
%           TB     -  total dissolved bor [mol/kg]
%           TH2S   -  total dissolved sulfide [mol/kg]
%           TNH4   -  total dissolved ammonium [mol/kg]
%           TPO4   -  total dissolved phosphate [mol/kg]
%           TSi    -  total dissolved silicate [mol/kg]
%           TSO4   -  total dissolved sulfate [mol/kg]
%           TF     -  total dissolved fluoride [mol/kg]
%
%       Output parameter: 
%           eqsys  -  structure array of pH and species equilibrium 
%                     concentrations [mol/kg]
%                     of the form ('name1',value1,'name2',value2,...)
%
%
%   The analytical solution for the pH is based on the mass action laws of:
%      CO2 + H2O <--> H+ + HCO3-        Kc1 = H * HCO3 / CO2
%      HCO3- <--> H+ + CO32-            Kc2 = H * CO3 / HCO3
%      B(OH)3 + H2O <--> H+ + B(OH)4-   Kb  = H * BOH4 / BOH3
%      H2S <--> H+ + HS-                Ks  = H * HS / H2S
%   and the corresponding equations for alkalinity and mass conservation:
%      TALK = HCO3 + 2*CO3 + BOH4 + HS
%      DIC = CO2 + HCO3 + CO3
%      TBOH4 = BOH4 + BOH3
%      TH2S = H2S + HS
%
%   The definition of TALK is a simplified form only considering the major
%   contributing species that allows for analytical solutions.
%
%   Equilibrium constants are calculated with KEQUILIB_SW and if the CO2
%   concentration exceeds 0.01 mol/kg KCO2_SW is used.
%
%   Note: The analytical solution of the 4th order polynomial produces
%   complex conjugate results and thus is solved using FZERO of MATLAB.
%   The analytical solution of the cubic or quadratic polynomials, i.e.
%   for TBOH4 = 0 and/or TH2S = 0, are solved according to the recipe
%   given in Bronstein & Semendjajew (1989) Taschenbuch der Mathematik.
%
%
%   Further speciations are calculated from:
%      H2O <--> H+ + OH-             Kw  = H * OH
%      NH4+ <--> H+ + NH3            Kn  = H * NH3 / NH4
%      H3PO4 <--> H+ + H2PO4-        Kp1 = H * H2PO4 / H3PO4
%      H2PO4- <--> H+ + HPO42-       Kp2 = H * HPO4 / H2PO4
%      HPO42- <--> H+ + PO43-        Kp3 = H * PO4 / HPO4
%      H4SiO4 <--> H+ + H3SiO4-      Ksi = H * H3SiO4 / H4SiO4
%      HSO4- <--> H+ + SO42-         Kso = H * SO4 / HSO4
%      HF <--> H+ + F-               Kf  = H * F / HF
%      TNH4 = NH4 + NH3
%      TPO4 = H3PO4 + H2PO4 + HPO4 + PO4
%      TSi = H4SiO4 + H3SiO4
%      TSO4 = HSO4 + SO4
%      TF = HF + F
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (February 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 9
     warning('PH_ANALYT:Input','PH_ANALYT: Missing input parameter! Please provide p,T,S,TA,DIC,TB,TS,TN,TP!')
end
if TA <= 0 && DIC <= 0
     error('PH_ANALYT:Input','PH_ANALYT: Please provide values for TA and DIC!')
end

% calculate equilibrium constants (use KCO2_SW for CO2 concentrations >0.01)
Kw  = kequilib_sw(press,temp,sal,'h2o');
Kc1 = kequilib_sw(press,temp,sal,'co2');
Kc2 = kequilib_sw(press,temp,sal,'hco3');
Kb  = kequilib_sw(press,temp,sal,'boh4');
Ks  = kequilib_sw(press,temp,sal,'h2s');
Kn  = kequilib_sw(press,temp,sal,'nh4');
Kp1 = kequilib_sw(press,temp,sal,'h3po4');
Kp2 = kequilib_sw(press,temp,sal,'h2po4');
Kp3 = kequilib_sw(press,temp,sal,'hpo4');
Ksi = kequilib_sw(press,temp,sal,'sio2');
Kso = kequilib_sw(press,temp,sal,'so4');
Kf  = kequilib_sw(press,temp,sal,'hf');
% calculate CO2 concentration from TA and DIC
p = Kc1*(1-DIC/TA);
q = Kc1*Kc2*(1-2*DIC/TA);
h1 = - p/2 + sqrt(p^2/4 - q);
h2 = - p/2 - sqrt(p^2/4 - q);
if isreal(h1) == 1 && h1 > 0
    H = h1;
else
    H = h2;
end
CO2 = DIC / (1 + Kc1/H + Kc1*Kc2/H^2);
if CO2 > .01
    Kw  = kco2_sw(press,temp,sal,'h2o');
    Kc1 = kco2_sw(press,temp,sal,'co2');
    Kc2 = kco2_sw(press,temp,sal,'hco3');
    Kso = kco2_sw(press,temp,sal,'so4');
    Kf  = kco2_sw(press,temp,sal,'hf');
end

% check the necessary order of the equation to be solved
if TB == 0 && TS == 0    % 0 = h^2 + p*h + q
    p = Kc1*(1-DIC/TA);
    q = Kc1*Kc2*(1-2*DIC/TA);
    h1 = - p/2 + sqrt(p^2/4 - q);
    h2 = - p/2 - sqrt(p^2/4 - q);
    if isreal(h1) == 1 && h1 > 0
        H = h1;
    else
        H = h2;
    end
elseif TB == 0 && TS > 0  % 0 = h^3 + r*h^2 + s*h + t
    r = Kc1*(1-DIC/TA) + Ks*(1-TS/TA);
    s = Kc1*Kc2*(1-2*DIC/TA) + Kc1*Ks*(1-(DIC+TS)/TA);
    t = Kc1*Kc2*Ks*(1-(2*DIC+TS)/TA);

%    H = fzero(@(h)h^3 + r*h^2 + s*h + t, [0 1]);

    % x = h + r/3 -> reduced equation: 0 = x^3 + k*x + l
    k = s - r^2/3;
    l = 2/27*r^3 - r*s/3 + t;
    % avoiding the complex numbers in the reduced solution
    rho    = sqrt(-k^3/27);
    phi = acos(-l/2/rho);
    h1 = 2*(rho)^(1/3) * cos(phi/3) - r/3;
    h2 = 2*(rho)^(1/3) * cos(phi/3 + 2*pi/3) - r/3;
    h3 = 2*(rho)^(1/3) * cos(phi/3 + 4*pi/3) - r/3;
    if isreal(h1) == 1 && h1 > 0
        H = h1;
    elseif isreal(h2) == 1 && h2 > 0
        H = h2;
    else
        H = h3;
    end
elseif TB > 0 && TS == 0  % 0 = h^3 + r*h^2 + s*h + t
    r = Kc1*(1-DIC/TA) + Kb*(1-TB/TA);
    s = Kc1*Kc2*(1-2*DIC/TA) + Kc1*Kb*(1-(DIC+TB)/TA);
    t = Kc1*Kc2*Kb*(1-(2*DIC+TB)/TA);

%    H = fzero(@(h)h^3 + r*h^2 + s*h + t, [0 1]);
    
    % x = h + r/3 -> reduced equation: 0 = x^3 + k*x + l
    k = s - r^2/3;
    l = 2/27*r^3 - r*s/3 + t;
    % avoiding the complex numbers in the reduced solution
    rho    = sqrt(-k^3/27);
    phi = acos(-l/2/rho);
    h1 = 2*(rho)^(1/3) * cos(phi/3) - r/3;
    h2 = 2*(rho)^(1/3) * cos(phi/3 + 2*pi/3) - r/3;
    h3 = 2*(rho)^(1/3) * cos(phi/3 + 4*pi/3) - r/3;
    if isreal(h1) == 1 && h1 > 0
        H = h1;
    elseif isreal(h2) == 1 && h2 > 0
        H = h2;
    else
        H = h3;
    end
else    % 0 = h^4 + b*h^3 + c*h^2 + d*h + e
   b = Kc1*(1-DIC/TA) + Kb*(1-TB/TA) + Ks*(1-TS/TA);
   c = Kc1*Kc2*(1-2*DIC/TA) + Kc1*Kb*(1-(DIC+TB)/TA) ...
       + Kc1*Ks*(1-(DIC+TS)/TA) + Kb*Ks*(1-(TB+TS)/TA);
   d = Kc1*Kc2*Kb*(1-(2*DIC+TB)/TA) + Kc1*Kc2*Ks*(1-(2*DIC+TS)/TA) ...
       + Kc1*Kb*Ks*(1-(DIC+TB+TS)/TA);
   e = Kc1*Kc2*Kb*Ks*(1-(2*DIC+TB+TS)/TA);

   H = fzero(@(h)h^4 + b*h^3 + c*h^2 + d*h + e, [0 1]);
   
%   % y = h + b/4 -> reduced equation: 0 = y^4 + p*y^2 + q*y + o
%   p = c;
%   q = b^3/8 - b*c/2 + d;
%   o = -3*b^4/256 + b^2*c/16 - b*d/4 + e;
%   % its cubic resolvent: 0 = z^3 + r*z^2 + s*z + t
%   r = 2*p;
%   s = p^2 - 4*o;
%   t = -q^2;
%   % x = z + r/3 -> reduced equation: 0 = x^3 + k*x + l
%   k = s - r^2/3;
%   l = 2/27*r^3 - r*s/3 + t;
%   % avoiding the complex numbers in the reduced solution
%   rho    = sqrt(-k^3/27);
%   phi = acos(-l/2/rho);
%   z1 = 2*(rho)^(1/3) * cos(phi/3) - r/3;
%   z2 = 2*(rho)^(1/3) * cos(phi/3 + 2*pi/3) - r/3;
%   z3 = 2*(rho)^(1/3) * cos(phi/3 + 4*pi/3) - r/3;
%   h1 = (sqrt(z1) + sqrt(z2) - sqrt(z3))/2 - b/4;   
%   h2 = (sqrt(z1) - sqrt(z2) + sqrt(z3))/2 - b/4;   
%   h3 = (-sqrt(z1) + sqrt(z2) + sqrt(z3))/2 - b/4;   
%   h4 = -(sqrt(z1) + sqrt(z2) + sqrt(z3))/2 - b/4;   
%    if isreal(h1) == 1 && h1 > 0
%        H = h1;
%    elseif isreal(h2) == 1 && h2 > 0
%        H = h2;
%    elseif isreal(h3) == 1 && h3 > 0
%        H = h3;
%    else
%        H = h4;
%    end
end    
% calculate species concentrations
pH     = -log10(H);
OH     = Kw / H;
CO2    = DIC / (1 + Kc1/H + Kc1*Kc2/H^2);
HCO3   = DIC / (1 + H/Kc1 + Kc2/H);
CO3    = DIC / (1 + H^2/Kc1/Kc2 + H/Kc2);
BOH4   = TB / (1 + H/Kb);
BOH3   = TB / (1 + Kb/H);
H2S    = TS / (1 + Ks/H);
HS     = TS / (1 + H/Ks);
NH4    = TN / (1 + Kn/H);
NH3    = TN / (1 + H/Kn);
H3PO4  = TP / (1 + Kp1/H + Kp1*Kp2/H^2 + Kp1*Kp2*Kp3/H^3);
H2PO4  = TP / (1 + H/Kp1 + Kp2/H + Kp2*Kp3/H^2);
HPO4   = TP / (1 + H^2/Kp1/Kp2 + H/Kp2 + Kp3/H);
PO4    = TP / (1 + H^3/Kp1/Kp2/Kp3 + H^2/Kp2/Kp3 + H/Kp3);
H4SiO4 = TSi / (1 + Ksi/H);
H3SiO4 = TSi / (1 + H/Ksi);
HSO4   = TSO4 / (1 + Kso/H);
SO4    = TSO4 / (1 + H/Kso);
HF     = TF / (1 + Kf/H);
F      = TF / (1 + H/Kf);

% final output
eqsys = struct('pH',pH,'H',H,'OH',OH,'TAlk',TA,...
    'DIC',DIC,'CO2',CO2,'HCO3',HCO3,'CO3',CO3,...
    'TBOH4',TB,'BOH4',BOH4,'BOH3',BOH3,...
    'TH2S',TS,'H2S',H2S,'HS',HS,...
    'TNH4',TN,'NH4',NH4,'NH3',NH3,...
    'TPO4',TP,'H3PO4',H3PO4,'H2PO4',H2PO4,'HPO4',HPO4,'PO4',PO4,...
    'TSi',TSi,'H4SiO4',H4SiO4,'H3SiO4',H3SiO4,...
    'TSO4',TSO4,'HSO4',HSO4,'SO4',SO4,...
    'TF',TF,'HF',HF,'F',F);
