function [pdisw,pdissw,pdispw] = phase_ch4sw(temp,S)

% PHASE_CH4SW -  calculates the phase boundaries of the binary system CH4-H2O
%
%     Usage: [pdisw,pdissw,pdispw] = phase_ch4sw(T,S)
%
%     Input parameters:
%         T       - temperature [degC]   (-10 < T < 30)
%         S       - practical salinity   (0 < S < 70)
%
%     Output parameters:
%         pdisw   - hydrate dissociation pressure for pure water [MPa]
%         pdissw  - hydrate dissociation pressure for seawater [MPa]
%         pdispw  - hydrate dissociation pressure for SO4-free porewater [MPa]
%
%     References:
%     Tishchenko et al. (2005) Chem.Geol. 219, 35-52.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%     Modification history: 
%       November 2009: correct function call implemented in the example (line 28)
%       September 2011: function name changed: phase_ch4h2o -> phase_ch4sw
%                       Header: parameter names in Usage corrected
%------------------------------------------------------------------------


%      Example: 
%       Plotting the phasediagram of CH4-seawater in the 
%       temperature range of -10 to +40 degC
%       at a salinity of 35.
%
%       clf;clear;hold on;
%       t=[-10:.01:40];
%       s=35;
%       [pdisw,pdissw,pdispw]=phase_ch4sw(t,s);
%       plot(t,pdissw,'k-','LineWidth',2);
%       set(gca,'YLim',[0 100]);
%       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
%       ylabel('Pressure / bar','FontSize',12,'FontWeight','bold');
%       title('Phase diagram of CH_4-H_2O','FontSize',14,'FontWeight','bold');
%       text(-80,30,'CH_4 hydrate','FontWeight','bold');
%       text(0,20,'CH_4 gas and liquid H_2O','FontWeight','bold');
%
%

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('PHASE_CH4SW:Input','PHASE_CH4SW: Missing input parameter! Please provide T and S!')
else



%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------
% check validity of data range. Validity range for temperature is an
% estimate (Tishchenko validated the formula for T=0-20C. The agreement 
% with experimental data up to 30 C is good. 

if ~any(S<0 | S>70 )&& ~any(temp<(-10) | temp>30) 
  
else
    bad = find(temp<(-10) | temp>30);
    temp(bad)=NaN*ones(size(bad));
    bad = find(S<0 | S > 70);
    S(bad)=NaN*ones(size(bad));
    warning('PHASE_CH4SW:OutOfParameterRange','PHASE_CH4SW: Input values out of range!')
    warning('PHASE_CH4SW:OutOfParameterRange','Equations valid for  T=(-10)-30 degC, S=0-70.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[T,S] = ndgrid(temp+273.15,S); % temperature in [K]

CL = S/1.80655; % chlorinity based on salinity
 
%------------------------------------------------------------------------
% Calculation of dissociation pressures of methane hydrate
%------------------------------------------------------------------------

% a) in pure water 
% after Handa (1990) J. Phys. Chem. 94, 2652-2657.
lnPdisw = -1208.1964 + 44097.00./T + 186.7594*log(T);
pdisw = exp(lnPdisw); % [MPa]
%
% b) in seawater (as function of absolute salinity)
lnPdissw = -1.6444866D3 - 0.1374178.*T + 5.4979866D4./T ...
    + 2.64118188D2.*log(T) + S.*1.005 .* (1.1178266D4 ...
    + 7.67420344.*T - 4.515213D-3.*T.^2 - 2.04872879D5./T ...
    - 2.17246046D3.*log(T)) + (S.*1.005).^2 .*(1.70484431D2 ...
    + 0.118594073.*T - 7.0581304D-5.*T.^2 ...
    - 3.09796169D3./T - 33.2031996.*log(T)) ;
pdissw = exp(lnPdissw); % [MPa]

%
% c) in anoxic porewater (i.e., Cso4=0)
lnPdispw = -1.6892692D3 - 0.15162984.*T + 5.60912482D4./T ...
    + 2.72067506D2.*log(T) + CL .* (2.06621298D4 ...
    + 14.18127.*T - 8.3417066D-3.*T.^2 - 3.78757519D5./T ...
    - 4.01544417D3.*log(T)) + CL.^2 .*(6.04018045D2 ...
    + 0.420253434.*T - 2.501548D-4.*T.^2 ...
    - 1.09745335D4./T - 1.17640966D2.*log(T)) ;
pdispw = exp(lnPdispw); % [MPa]

end
