function pvco2 = pp2pv_co2(press,temp,pco2)

% PP2PV_CO2 - converts the partial pressure of CO2 in a gas mixture into
%     partial volume
%
%     Usage: pv = pp2pv_co2(p,T,pp)
%
%     Input parameter (p,T: scalars or vectors; pp: scalar or vector of same size as p): 
%           p      - overall pressure [MPa]        (0.1 < p < 800)
%           T      - temperature [degC]            (-57 < T < 827)
%           pp     - partial pressure of CO2 [MPa] (0 < pp < p)
%           
%     Output parameter: 
%           pv     - partial volume of CO2 [ppmV]
%
%     Comments: 
%     Equations are only valid for pressures below the vapor pressure of CO2.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (May 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       December 2012: handling for size(pch4) ~= size(press) implemented
%       June 2013: checks for input and parameter range implemented
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('PP2PV_CO2:Input','PP2PV_CO2: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>800) && ~any(temp< -57 | temp>827)
else
    bad = find(press<0 | press > 800);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-57 | temp>827);
    temp(bad)=NaN*ones(size(bad));
    warning('PP2PV_CO2:OutOfParameterRange','PP2PV_CO2: Input values out of range!')
    warning('PP2PV_CO2:OutOfParameterRange','Equations valid for p = 0-800 MPa, T=(-57) - 827 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P] = ndgrid(press,temp+273.15); % temperature in [K]

rho = eos_co2(press,temp); % density of CO2 at total pressure

if size(press) == size(pco2)
    rhop = eos_co2(pco2,temp); % density of CO2 at partial pressure
else
    rhop2 = eos_co2(pco2,temp); % density of CO2 at partial pressure
    rhop = zeros(length(press),length(temp));
    for i=1:length(press)
        rhop(i,:) = rhop2(:,:);
    end
end

pvco2 = rhop./rho; % volume fraction of CO2 in gas phase
pvco2 = pvco2.*1d6; % partial volume in ppmV

% check, if CO2 is liquid at given pT conditions 
    [~,~,pv] = phase_co2sw(temp,0);
    for i = 1:length(temp)
            for k = 1:length(press)
                if P(k,i) > pv(i) || isnan(pv(i))
                    pvco2(k,i) = NaN;
                    warning('PP2PV_CO2:OutOfParameterRange','Equations valid for p < p_vap!')
                end
            end
    end

end