function pvo2 = pp2pv_o2(press,temp,po2)

% PP2PV_O2 - converts the partial pressure of O2 in a gas mixture into
%     partial volume
%
%     Usage: pv = pp2pv_o2(p,T,pp)
%
%     Input parameter (p,T: scalars or vectors; pp: scalar or vector of same size as p): 
%           p      - overall pressure [MPa]        (0 < p < 81.8)
%           T      - temperature [degC]            (-219 < T < 127)
%           pp     - partial pressure of O2 [MPa]  (0 < pp < p)
%           
%     Output parameter: 
%           pv     - partial volume of O2 [ppmV]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('PP2PV_O2:Input','PP2PV_O2: Missing input parameter! Please provide p, T and pp!')
else

    
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<=0 | press>81.8) && ~any(temp< -219 | temp>127)
else
    bad = find(press<=0 | press > 81.8);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-219 | temp>127);
    temp(bad)=NaN*ones(size(bad));
    warning('PP2PV_O2:OutOfParameterRange','PP2PV_O2: Input values out of range!')
    warning('PP2PV_O2:OutOfParameterRange','Equations valid for p = 0-81.8 MPa (p>0), T=(-219) - 127 degC.')
end
    
rho = eos_o2(press,temp); % density of O2 at total pressure

if size(press) == size(po2)
    rhop = eos_o2(po2,temp); % density of O2 at partial pressure
else
    rhop2 = eos_o2(po2,temp); % density of O2 at partial pressure
    rhop = zeros(length(press),length(temp));
    for i=1:length(press)
        rhop(i,:) = rhop2(:,:);
    end
end

pvo2 = rhop./rho; % volume fraction of O2 in gas phase
pvo2 = pvo2.*1d6; % partial volume in ppmV

end