function mCH4 = solu_ch4co2(press,temp)

% SOLU_CH4CO2 -  calculates the solubility of CH4 in (liquid) CO2 by a
%      "brute force" strategy (slow!): The feed composition is varied 
%      until the density as calculated by a Peng Robinson EOS is
%      returned as a 2-component vector indicating a 2-phase system.
%
%      Usage: m = solu_ch4co2(p,T)
% 
%      Input parameters (scalars!): 
%           p       - pressure [MPa]
%           T       - temperature [degC] 
%
%      Output parameter:
%           m       - solubility of CH4 in liquid CO2 [mol/kg]             
%
%      References:
%      Al-Sahhaf et al. (1983) Ind.Eng.Chem.Fund. 22, 372-380.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------
 

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------


if nargin < 2
     warning('SOLU_CH4CO2:Input','SOLU_CH4CO2: Missing input parameter! Please provide p and T!')
else
   

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

molN2 = 0;
f = [1 1];
n = 0;

%------------------------------------------------------------------------
% search transition from 1 phase to 2 phases
%------------------------------------------------------------------------

while f(2) == f(1) && n < 1002
    molCO2 = 1 - (n - 1)/1000;
    molCH4 = (n-1)/1000;
     rho = density_ch4co2n2(press,temp,molCH4,molCO2,molN2);
     if length(rho) == 2
         if abs((rho(1)- rho(2))/rho(1)) < 1d-5
             rho = rho(1);
         end
         % handle erratic 2 phase results
        molCO22 = 1 - (n )/1000;
        molN22 = (n)/1000;
        rho2 = density_ch4co2n2(press,temp,molCH4,molCO22,molN22);
    if length(rho2) == 1
        rho = rho(1);
    end
     end
     f(1) = f(2);
     f(2) = length(rho);
     n = n+1;
end

%------------------------------------------------------------------------
% If no transition is found, only 1 phase with infinite solubility can
% exist (gas, supercritical). If a phase split is detected, set mCH4 to
% molality just before phase split
%------------------------------------------------------------------------

if n == 1001
    mCH4 = Inf;
else 
    molCH4 = (n-2)/1000;
    molCO2 = 1 - (n - 2)/1000;
    mCH4 = molCH4/molCO2/44.01d-3;
end

end 
    
% alternative strategy to calculate solubility: faster, but if the gridding is
% too coarse, it might miss the liquid region
%
% molCO2 = 1;
% molCH4 = (0:1:10);
% phases = zeros(11,1);
%
% for i = 1:5
% 
%     for j = 1:11
%         rho = density_ch4co2n2(press,temp,molCH4(j),molCO2,molN2);
%         l = length(rho);
%         phases(j)=l;
%     end
%     f = find(phases > 1);
%     if  length(f) < 2
%         p1 = molCH4(1);
%         p2 = molCH4(2);
%     else
%         p1 = molCH4(f(1)-1);
%         p2 = molCH4(f(1));
%     end
%         molCH4 = (p1:1*10^(-i):p2);
%    
% end
% 
% if isempty(f)
%     mCH4 = NaN;
% else
%     mCH4 = p1/44.01d-3;
% end