function [mch4sw,mch4pw] = solu_ch4gh(press,temp,sal)

% SOLU_CH4GH -  calculates the solubility of CH4 in seawater and
%     SO4-free porewater in equilibrium with methane hydrate (no gas phase)
%
%     Usage:   [mch4sw,mch4pw] = solu_ch4gh(p,T,S)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (0.1 < P < 50)
%           T        -  temperature [degC] (0 < T < 20)
%           S        -  practical salinity (S < 70)
%     
%     Output parameter: 
%           mch4sw    - molality of CH4 in seawater [mol/kg]
%           mch4pw    - molality of CH4 in SO4-free porewater [mol/kg]
%
%     References:
%     Tishchenko et al. (2005) Chem. Geol. 219, 35-52.
%                
%     Comments: 
%     This script does not check, if the pressure and temperature
%     conditions are outside the hydrate stability zone. Use solu_ch4 to
%     discriminate between 2-phase and 3-phase solubilities.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history:
%       September 2011: variable names changed: 
%          ch4ghsw -> mch4sw; ch4ghpw -> mch4pw; phase_ch4h2o -> phase_ch4sw
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CH4GH:Input','SOLU_CH4GH: Missing input parameter! Please provide p, and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>50) && ~any(temp<0 | temp>20) ...
        && ~any( sal<0 | sal>70)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>20);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>70);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_CH4GH:OutOfParameterRange','SOLU_CH4GH: Input values out of range!')
    warning('SOLU_CH4GH:OutOfParameterRange','Equations valid for p = 0.1-50 MPa, T= 0-10 degC, S = 0-70.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------


[P,T,S] = ndgrid(press,temp+273.15,sal); % P [MPa], T [K]

% call external function to calculate the phase boundaries
[~,pdissw,pdispw] = phase_ch4sw(temp,sal);  

% resize matrices to account for pressure vector
Pdissw= zeros(size(P));
Pdispw= zeros(size(P));

% create matrices for phase boundaries
for i=1:size(press,2)
Pdissw(i,:,:) = pdissw(:,:); 
Pdispw(i,:,:) = pdispw(:,:); 
end

CL = S./1.80655; % chlorinity based on practical salinity


%------------------------------------------------------------------------
% Calculation of solubilities
%------------------------------------------------------------------------


% a) in seawater 
% 1. at the hydrate dissociation pressure Pdissw
% (-> for 3 coexisting phases: gas, hydrate, seawater)
lnch4ghswPdis = -2.5640213D5 - 1.6448053D2.*T ...
    + 9.1089042D-2.*T.^2 + 4.90352929D6./T ...
    + 4.93009113D4.*log(T) + S .* (-5.16285134D2 ...
    - 0.33622376.*T + 1.88199047D-4.*T.^2 ...
    + 9.76525718D3./T + 9.9523354D1.*log(T)); 
%
% 2. at hydrostatic pressure
% (-> seawater in equilibrium with methane hydrate; no gas)
lnch4ghsw = lnch4ghswPdis + ...
    (P-Pdissw) .* (5.04597D-2 + 7.64415D-4.*S ...
    - T .* (3.90236D-4 + 5.48947D-6.*S) ...
    + T.^2 .* (7.06154D-7 + 9.87742D-9.*S)) ...
    + (P-Pdissw).^2 .* (7.57285D-5 - 1.90867D-8.*S ...
    - 1.4483D-10.*S.^2 - T .* (1.96207D-7 - 6.67456D-11.*S)); 
mch4sw = exp(lnch4ghsw); % [mol/kg]
%
% b) in anoxic porewater (i.e., Cso4=0)
% 1. at the hydrate dissociation pressure Pdissw
% (-> for 3 coexisting phases: gas, hydrate, porewater)
lnch4ghpwPdis = -2.54350858D5 - 1.630680662D2.*T ...
    + 9.0255483D-2.*T.^2 + 4.86599512D6./T ...
    + 4.89020747D4.*log(T) + CL .* (-1.10117464D3 ...
    - 0.72234127.*T + 4.07255D-4.*T.^2 ...
    + 2.07425019D4./T + 2.12503801D2.*log(T)); 
% 
% 2. at hydrostatic pressure
% (-> porewater in equilibrium with methane hydrate; no gas)
lnch4ghpw = lnch4ghpwPdis + ...
    (P-Pdispw) .* (5.04597D-2 + 1.38095D-3.*CL ...
    - T .* (3.90236D-4 + 9.917D-6.*CL) ...
    + T.^2 .* (7.06154D-7 + 1.78441D-8.*CL)) ...
    + (P-Pdispw).^2 .* (7.57285D-5 - 3.44811D-8.*CL ...
    - 4.72671D-10.*CL.^2 - T .* (1.96207D-7 - 1.20579D-10.*CL)) ;
mch4pw = exp(lnch4ghpw); % [mol/kg]

end
