function pp = solu_co2ch4(press,temp,sal)

% SOLU_CO2CH4 -  calculates the partial pressure of CO2 in a methane gas 
%     bubble in seawater
%     
%     Usage: pp = solu_co2ch4(p,T,S)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (3.5 < P < 50)
%           T        -  temperature [degC] (0 < T < 25) 
%           S        -  practical salinity (S < 40)
%     
%     Output parameter: 
%           pp       - partial pressure of CO2 in CH4-dominated gas [MPa]
%  
%     References:
%     Wong et al. (2005) J.Chem.Eng.Data 50, 822-831.
%     Tishchenko et al. (2009) 8th International CO2 Conference, Jena.
%     Duan et al. (2008) Energy & Fuels 22, 1666-1674.
%
%     Comments: 
%     If the pTS values are within the range of gaseous CO2, the function 
%     output is set to NaN.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       November 2012: syntax changed to Matlab2012 style
%                      indexing error corrected in check for NaN (before
%                      fsolve is called)
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CO2CH4:Input','SOLU_CO2CH4: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<3.5 | press>50) && ~any(temp<0 | temp>25) ...
        && ~any( sal<0 | sal> 40)
else
    bad = find(press<3.5 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>25);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_CO2CH4:OutOfParameterRange','SOLU_CO2CH4: Input values out of range!')
    warning('SOLU_CO2CH4:OutOfParameterRange','Equations valid for p = 3.5-50 MPa, T= 0-25 degC, S = 0-263.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*1e6,temp+273.15,sal*1.005); % P [Pa], T [K], absolute salinity
P0 = 0.1; % reference pressure [MPa]
R = 8.314467; % gas constant [J/mol/K]

% call external function to calculate vapor pressure of CO2
[~,~,Pv_1] = phase_co2sw(temp,sal); 
Pv_1 = Pv_1 .*1e6; % Pv in [Pa]
% resize matrix
Pv = zeros(size(P));
for i=1:size(P,1)
    Pv(i,:,:)=Pv_1;
end

%------------------------------------------------------------------------
% calculation of dissociation pressure
%------------------------------------------------------------------------

lnpdis = -1.5902671e3 - 1.66823e2.*S + 1.82015 .* S.^2 + ...
    (1.762994e1 + 1.811196 .* S - 1.96512e-2 .* S.^2) .* T + ...
    (-6.499176e-2 - 6.555242e-3 .*S + ...
    7.0722e-5 .*S.^2).*T.^2 + (8.03617e-5 + 7.909328e-6 .* S - ...
    8.48414e-8 .*S.^2).*T.^3;
pdis = exp(lnpdis); % [Pa]

%------------------------------------------------------------------------
% calculation of dissolved CO2 from liquid phase
%------------------------------------------------------------------------

lnm_l = -3.29489e1 - 3.41348e-2.*T + 5.380e-6.*T.^2 + 5.180547e3./T + ...
    2.33585e1.*log(T/100) + S.*(-1.1694e-2 - 1.71518e-4 .*T + ...
    4.518e-7.*T.^2 + 5.5246./T + (-1.7667e-10 + 2.93886e-13 .*T + ...
    2.36595e-8 ./T).*P) + (-2.78153e-7 + 5.532929e-10 .* T + ...
    3.70381e-5 ./T).*P + (3.00159e-15 - 5.8187e-18 .*T - 3.9493e-13 ./T)...
    .*P.^2;
m_l = exp(lnm_l); % [mol/kg]

% check, if PTS values are within gas phase of CO2, 
decide = P < Pv;
% set m_l to NaN for gas phase
m_l(decide) = NaN;

%------------------------------------------------------------------------
% calculation of dissolved CO2 in equilibrium with hydrate phase
%------------------------------------------------------------------------

% at dissociation pressure:
lnmh_pdis = 1.416202e1 - 8.01074.*S.^(0.5) + 3.791107 .* S - ...
    3.023329e-1 .* S .^(1.5) + ( -1.661977e-1 + 5.66596e-2 .* S.^(0.5) - ...
    2.7240266e-2 .* S + 2.1723465e-3 .* S.^(1.5)) .* T + (4.15936e-4 - ...
    9.9918e-5 .* S.^(0.5) + 4.88634e-5 .* S - 3.89824e-6 .* S.^(1.5)) .* T.^2;

% at pressure P
lnmh = lnmh_pdis + (-9.193e-9 - 5.24e-12 .*S + (2.68e-11 + 2.34e-14 .*S)...
    .*T).*(P - pdis) + (7.888e-17 + 3.61e-20 .* S + (-2.114e-19 - ...
    1.4947e-22.*S).*T).*(P - pdis).^2;
m_h = exp(lnmh); % [mol/kg]

%------------------------------------------------------------------------
% calculation of dissolved CO2 from gas phase (bubbles)
%------------------------------------------------------------------------

m_g = zeros(size(P));

% calculation of pure gas fugacity coefficient phi
[~, ~, ~, ~, ~, ~, phi] = eos_co2(press,temp);

%------------------------------------------------------------------------
% Henry constant
%------------------------------------------------------------------------

K0 = exp(-5.97508e1 + 9.34517e3./T + 2.33585e1 .* log(T./100) - ...
    3.2874e-3 .* T + 5.38e-6 .*T.^2 + S .*(2.2843e-2 - 2.2534e-4 .* T ...
    + 4.518e-7 .* T.^2)); %mol/kg/bar

%------------------------------------------------------------------------
% Interaction parameter lambda
%------------------------------------------------------------------------

lambda = -0.2546 + 75.571./T - 5.428e-10 .*P;

%------------------------------------------------------------------------
% calculation of the molar volume Vh2o of pure water [cm^3/mol] [Duan2008]
%------------------------------------------------------------------------
P2 = P.*1e-6; % pressure in MPa

k01 = 0.000000327225;
k02 = -0.00042095;
k03 = 0.232594;
k04 = -41.692;
k05 = 5712.92;
k11 = -0.000000000232306;
k12 = 0.000000291138;
k13 = -0.000149662;
k14 = 0.0359860;
k15 = -3.55071;
k21 = 0.0000000000000257241;
k22 = -0.0000000000124336;
k23 = 0.000000542707;
k31 = -0.00000000000000000442028;
k32 = 0.00000000000000210007;
k33 = -0.0000000000811491;

k0 = k01.*T.^3 + k02.*T.^2 + k03.*T + k04 + k05./T;
k1 = k11.*T.^3 + k12.*T.^2 + k13.*T + k14 + k15./T;
k2 = k21.*T.^3 + k22.*T.^2 + k23;
k3 = k31.*T.^3 + k32.*T.^2 + k33;

% pressure is required in bar
Vh2o = k0 + k1.*P2.*10 + k2.*(P2.*10).^2 + k3.*(P2.*10).^3; % [cm^3/mol]

%------------------------------------------------------------------------
% calculation of the apparent molar volume Vco2 of CO2 in pure water
% [cm^3/mol][Duan2008]
%------------------------------------------------------------------------

A11 =  0.38384020e-3; 
A21 = -0.57709332e-5;
A12 = -0.55953850; 
A22 =  0.82764653e-2;
A13 =  0.30429268e3; 
A23 = -0.43813556e1;
A14 = -0.72044305e5; 
A24 =  0.10144907e4;
A15 =  0.63003388e7; 
A25 = -0.86777045e5;

A1 = A11.*T.^2 + A12.*T + A13 + A14./T + A15./T.^2;
A2 = A21.*T.^2 + A22.*T + A23 + A24./T + A25./T.^2;

Vco2 = Vh2o .* (1 + A1 + A2.*P2); %[cm^3/mol]

%------------------------------------------------------------------------
% calculation of CO2 dissolved from gas phase by numerical iteration
%------------------------------------------------------------------------

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

% call minimizing function
for i=1:size(P,1)
    for j=1:size(P,2)
        for k = 1:size(P,3)
         % make sure that input value for P, T and S is a number
        if isnan(P(i,j,k)) || isnan(T(i,j,k)) || isnan(S(i,j,k)); % make sure that input value for P, T and S is a number
            m_g(i,j,k) = NaN;
        else  
            m_g(i,j,k)= fsolve(@(m) func(m,P(i,j,k),T(i,j,k),phi(i,j),K0(i,j,k),Vco2(i,j,k),P0,R,lambda(i,j,k)),m_l(i,j,k),options);
        end
        end
    end
end


%------------------------------------------------------------------------
% calculation of CO2 volume fraction
%------------------------------------------------------------------------

mf_l = m_l./m_g; % CO2 fraction in liquid phase
mf_h = m_h./m_g; % CO2 fraction in hydrate phase

% check, if pressure is above or below hydrate dissociation pressure, set
% CO2 fraction to corresponding value
decide = P < pdis;
mf = mf_h;
mf(decide) = mf_l(decide); 

pp = mf.*P.*1e-6; % partial pressure in MPa

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = func(m,P,T,phi,K0,Vco2,P0,R,lambda)
% minimization function to calculate concentration of CO2 dissolved from
% gas phase

f = log(phi.*P.*1e-5.*K0) - Vco2.*(P.*1e-6 - P0)./R./T - ...
    2.*lambda.*m - log(m);
