function mCO2sw = solu_co2gl(press,temp,sal) 
% SOLU_CO2GL -  calculates the CO2 equilibrium concentration in seawater
%     with respect to gas and liquid phase
%
%     Usage: m = solu_co2gl(p,T,S)
%
%     Input parameters (scalars or vectors): 
%         p        -  pressure [MPa]     (0.1 < P < 100)
%         T        -  temperature [degC] (0 < T < 162)
%         S        -  practical salinity (S < 263)
%     
%     Output parameter: 
%         m        -  molality of CO2 in seawater [mol/kg]
%
%     References: 
%     Duan & Sun (2003) Chemical Geology 193, 257-271.           
%     Duan et al. (2006) Marine Chemistry 98, 131-139.   
%
%     Comments: 
%     This script does not check, if the pressure and temperature
%     conditions are inside the hydrate stability zone. Use solu_co2 to
%     discriminate between 2-phase and 3-phase solubilities.
%
%     Deviation of solubility values from those of Wong et al. (2005)
%     (J.Chem.Eng.Data 50, 822-831.) is <5% for pressures up to 250 bar.
%
%     No dissociation pressure for pore water available so far.
%     No solubility of liquid/gaseous CO2 in SO4-free porewater available so far.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------
 
% ------------------------------------------------------------------
%   Modification history: 
%       November 2009: 	Array size of Pv corrected (line 75)
%       November 2009: 	Unit for temperature corrected in comments (l.195-197)
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       November 2012: 	syntax changed to Matlab2012 style
%       February 2013: 	function call for seasalt updated
%       June 2013:	gamma term corrected
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CO2GL:Input','SOLU_CO2GL: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<0 | temp>162) ...
        && ~any( sal<0 | sal> 263)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>162);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>263);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_CO2GL:OutOfParameterRange','SOLU_CO2GL: Input values out of range!')
    warning('SOLU_CO2GL:OutOfParameterRange','Equations valid for p = 0.1-100 MPa, T= 0-162 degC, S = 0-70.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*10,temp+273.15,sal); % pressure in bar

% call external function to calculate the phase boundaries
[~,~,pv] = phase_co2sw(temp,sal); 

% resize matrices to account for pressure vector
Pv = P;
for i=1:size(P,1) % create matrices for phase boundaries
Pv(i,:,:) = pv(:,:).*10; % vapor pressure in bar
end

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

%------------------------------------------------------------------------
% Calculation of chemical potential (uco2), Pitzer parameters lambda and zeta
% fugacity coefficient of CO2 (phico2) and saturation pressure of water 
% from multiparametric equations.
%------------------------------------------------------------------------

[uco2] = par1(P,T); % mu/RT
[lambdaNa] = par2(P,T);
[zetaNaCl] = par4(P,T);
[phico2] = fcco2(P,T,Pv);
[psh2o] = satpress(T);

%------------------------------------------------------------------------
% Calculation of molality of CO2 in solution 
%------------------------------------------------------------------------

xco2sw = (P - psh2o)./P ;% mole fraction of CO2 in gas phase
lnmco2 = log(xco2sw.*phico2.*P) - uco2 ...
        - 2.*lambdaNa.*(mNa + mK + 2.*mCa + 2.*mMg) ...% molalities in mol/kg
        + 0.07.*mSO4 - zetaNaCl.*(mNa + mK + 2.*mCa + 2.*mMg).*(mCl); 
  
mCO2sw = exp(lnmco2); % Molality [mol/kg]

end
% ----------------------------------------------------------

function [uco2] = par1(P,T)
% parametric calculation of the chemical potential


c1 = 28.9447706;
c2 = -0.0354581768;
c3 = -4770.67077;
c4 = 1.02782768e-5;
c5 = 33.8126098;
c6 = 9.0403714e-3;
c7 = -1.14934031e-3;
c8 = -0.307405726;
c9 = -0.0907301486;
c10 = 9.32713393e-4;
c11 = 0;

d1 = 134.72067;
d2 = -3.6727291e-1;
d3 = -14132.405;
d4 = 4.7809063e-4;
d5 = -5622.8080;
d6 = 7.9181559e-2;
d7 = -1.2283602e-2;
d8 = -2.9597665;
d9 = 6.5155997e-1;
d10 = 7.4901468e-4;
d11 = 0;

decide = T < 290;

uco2 = c1 + c2.*T + c3./T + c4.*T.^2 + c5./(630 - T) + c6.*P ...
     + c7.*P.*log(T) + c8.*P./T + c9.*P./(630 - T)...
     + c10.*P.^2./(630 - T).^2 + c11.*T.*log(P);

uco2(decide) = d1 + d2.*T(decide) + d3./T(decide) + d4.*T(decide).^2 + d5./(630 - T(decide)) + d6.*P(decide) ...
     + d7.*P(decide).*log(T(decide)) + d8.*P(decide)./T(decide) + d9.*P(decide)./(630 - T(decide))...
     + d10.*P(decide).^2./(630 - T(decide)).^2 + d11.*T(decide).*log(P(decide));
 
% ----------------------------------------------------------

function [lambdaNa] = par2(P,T)
% parametric calculation of interaction parameter lambda

c1 = -0.411370585;
c2 = 0.000607632013;
c3 = 97.5347708;
c8 = -0.0237622469;
c9 = 0.0170656236;
c11 = 0.0000141335834;

lambdaNa = c1 + c2.*T + c3./T + c8.*P./T + c9.*P./(630 - T)...
     + c11.*T.*log(P);


% ----------------------------------------------------------

function [zetaNaCl] = par4(P,T)
% parametric calculation of interaction parameter zeta

c1 = 0.000336389723;
c2 = -0.0000198298980;
c8 = 0.0021222083;
c9 = -0.00524873303;

zetaNaCl = c1 + c2.*T + c8.*P./T + c9.*P./(630 - T);

% ----------------------------------------------------------


function [phico2] = fcco2(P,T,Pv)

% Calculation of the fugacity coefficient of CO2 
% equations valid for P < 100 MPa and 273 < T < 435)

% case discrimination: set P1 value depending on T

phico2= zeros(size(P,1),size(P,2),size(P,3));

decide1 = T < 304;
decide2 = T <405;

P1 = 200 * ones(size(P,1),size(P,2),size(P,3));% for T> 405 K
P1(decide2) = 75 + (T(decide2)-305).*1.25; % for 305 < T < 405 K
P1(decide1) = Pv(decide1); % for T < 305 K

for i=1:size(P,1)
    for j=1:size(P,2)
        for k=1:size(P,3)

if (P(i,j,k) < P1(i,j,k))

c1 = 1;
c2 = 4.7586835e-3;
c3 = -3.3569963e-6;
c4 = 0;
c5 = -1.3179396;
c6 = -3.8389101e-6;
c7 = 0;
c8 = 2.2815104e-3;
c9 = 0;
c10 = 0;
c11 = 0;
c12 = 0;
c13 = 0;
c14 = 0;
c15 = 0;

elseif (P(i,j,k) >= P1(i,j,k)) && (T(i,j,k) < 340)

c1 = -0.71734882;
c2 = 0.00015985379;
c3 = -0.00000049286471;
c4 = 0;
c5 = 0;
c6 = -0.00000027855285;
c7 = 0.0000000011877015;
c8 = 0 ;
c9 = 0 ;
c10 = 0 ;
c11 = 0;
c12 = -96.539512;
c13 = 0.44774938;
c14 = 101.81078;
c15 = 0.0000053783879;

else % (if (P >= P1) && (340 <= T) && (T < 435)

c1 = 5.0383896;
c2 = -0.0044257744;
c3 = 0;
c4 = 1.9572733;
c5 = 0;
c6 = 0.0000024223436;
c7 = 0;
c8 = -0.00093796135;
c9 =  -1.5026030;
c10 = 0.0030272240;
c11 = -31.377342;
c12 = -12.847063;
c13 = 0;
c14 = 0;
c15 = -0.000015056648;

end

phico2(i,j,k) = c1 + (c2 + c3.*T(i,j,k) + c4./T(i,j,k) + c5./(T(i,j,k)-150)).*P(i,j,k) ...
    + (c6 + c7.*T(i,j,k) + c8./T(i,j,k)).*P(i,j,k).^2 + (c9 + c10.*T(i,j,k) + c11./T(i,j,k)).*log(P(i,j,k))...
     + (c12 + c13.*T(i,j,k))./P(i,j,k) + c14./T(i,j,k) + c15.*T(i,j,k).^2;
    
        end
    end
end

% ------------------------------------------------------------


function [psh2o] = satpress(T)

%%% calculation of saturation pressure of water [bar]

Pc = 220.85; % in bar
Tc = 647.29;
TT = (T-Tc)./Tc;
c1 = -38.640844;
c2 = 5.8948420;
c3 = 59.876516;
c4 = 26.654627;
c5 = 10.637097;

    
psh2o = Pc.*T./Tc.*(1 + c1.*(-TT).^1.9 + c2.*TT + c3.*TT.^2 ...
    + c4.*TT.^3 + c5.*TT.^4);   % pressure in bar 


% ------------------------------------------------------------
