function mNaCl = solu_nacl(press,temp)

% SOLU_NACL -  calculates the saturation concentration of NaCl in seawater
%       from a multiparametric equation
%
%       Usage: m = solu_nacl(p,T)
%
%       Input parameters (vectors or scalars): 
%           p     - pressure [MPa]       (0 < P < 500)
%           T     - temperature [degC]   (0 < T < 1000)
%
%       Output parameter:
%           m     - molality of NaCl at saturation [mol/kg]
%
%       References:
%       Driesner & Heinrich (2007) GCA 71, 4880-4901.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('SOLU_NACL:Input','SOLU_NACL: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if  ~any(temp<(0) | temp>1000) && ~any(press<(0) | press>500)

else
    bad = find(temp<(0) | temp>1000);
    temp(bad)=NaN*ones(size(bad));
    bad = find(press<(0) | press>500);
    press(bad)=NaN*ones(size(bad));

 warning('SOLU_NACL:OutOfParameterRange','SOLU_NACL: Input values out of range!')
 warning('SOLU_NACL:OutOfParameterRange','Equations valid for p=1-500 MPa, T=0-1000 degC.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P,T] = ndgrid(press*10,temp); % pressure in bar

mmH2O = 18.015;  % molar mass of H2O
TtNaCl = 800.7; % triple point temperature of NaCl [C]
PtNaCl = 5e-4; % triple point pressure of NaCl [bar]

c1 = 0.0989944 + 3.30796e-6.*P - 4.71759e-10.*P.^2;
c2 = 0.00947257 - 8.66460e-6.*P + 1.69417e-9.*P.^2;
c3 = 0.610863 - 1.51716e-5.*P + 1.19290e-8.*P.^2;
c4 =-1.64994 + 2.03441e-4.*P - 6.46015e-8.*P.^2;
c5 = 3.36474 - 1.54023e-4.*P + 8.17048e-8.*P.^2;
c6 = 1.0 - c1 - c2 - c3 - c4 - c5;

%------------------------------------------------------------------------
% Calculation of melting temperature for given pressures
%------------------------------------------------------------------------

Tm = TtNaCl + 2.4726e-2.*(P - PtNaCl);

% check, if any temperature is above the corresponding melting temperature

T_test = Tm-T;
if any (T_test < 0)
    warning('SOLU_NACL:OutOfParameterRange','Liquid saturation is not a valid concept for temperatures below the melting curve.')
    warning('SOLU_NACL:OutOfParameterRange','Higher temperatures are set to NaN')
    T(T_test < 0) = NaN;
end

%------------------------------------------------------------------------
% Calculation of saturation mole fraction xNaCl
%------------------------------------------------------------------------

xNaCl = c1 + c2.*(T./Tm) + c3.*(T./Tm).^2 + c4.*(T./Tm).^3 + ...
        c5.*(T./Tm).^4 + c6.*(T./Tm).^5;

    mNaCl = xNaCl.*1000./(1 - xNaCl)./mmH2O;
end 