function [m, KH, KHx] = soluhenry_co2sw(press,temp,sal,pp)

% SOLUHENRY_CO2SW -  calculates the equilibrium concentration of CO2
%     in seawater from Henry's law
%   
%     Usage:  [m KH KHx]  = soluhenry_co2sw(p,T,S,pp)
%
%     Input parameters (scalars or vectors, pp: scalar or matrix of size p x T x S): 
%           p        -  pressure [MPa]     (0.1 < p < 35 or gas/liquid phase boundary)
%           T        -  temperature [degC] (0 < T < 162)
%           S        -  practical salinity (S < 263)
%           pp       -  partial pressure of CO2 [MPa] (pp < p)
%                       pp is set to p if no input value is provided 
%     
%     Output parameter: 
%           m        - molality of CO2 in seawater [mol/kg]
%           KH       - pressure and salinity corrected Henry constant [mol/kg/MPa]
%           KHx      - pressure and salinity corrected Henry constant [1/MPa]
%
%     References:
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535. (vapor pressure of water)
%     Duan et al. (2006) Mar.Chem. 98, 131-139. (gamma term)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of CO2)
%
%     Comments: 
%     The solubility of CO2 in (sea)water as function of pressure, 
%     temperature and salinity is related to Henry's constant KH 
%     by the equation: 
%      m = f*KH0(T)/(gamma(S,T)*exp(integral(V_mol(P,T)/R/T) dP)))
%        f:     fugacity
%        gamma: activity coefficient, 
%        V_mol: apparent molar volume of dissolved gas
%
%     Henry's law is not well suited for high pressures and should be used
%     with care at high pressures: Keep in mind that results are expected
%     to be inaccurate.
%          
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_CO2SW:Input','SOLUHENRY_CO2SW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>50) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>40)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_CO2SW:OutOfParameterRange','SOLUHENRY_CO2SW: Input values out of range!')
    warning('SOLUHENRY_CO2SW:OutOfParameterRange','Equations valid for p = 0.1-50 MPa, T= 0-80 degC, S = 0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press,temp+273.15,sal); % temperature in [K]

% check, if CO2 is liquid at given pT conditions 
    [~,~,pv] = phase_co2sw(temp,0);
    for i = 1:length(temp)
            for k = 1:length(press)
                if P(k,i,1) > pv(i) || isnan(pv(i))
                    P(k,i,:) = NaN;
                    warning('SOLUHENRY_CO2SW:OutOfParameterRange','Equations valid for p < p_vap!')
                end
            end
    end

   if nargin == 3 % if no partial pressure is provided by user, partial pressure is set to system pressure
       pp = P; 
   end

% CO2 specific constants
a = -2.3d9;
b = 1.69d-3;
N = 1.29;
sigma12 = 3.0d-8; % [cm]
ek12 = 650;

Omega = 55.51d-6; % 
c1 = 0.0033; % [m^3/kg]
c2 = 0.002; %[m^3/kg]
sigma11 = 2.561d-8; % [cm]
ek11 = 1255;
lambda = 1.22;
Na = 6.02215d23; % Avogadro's number
R = 8.3145; % gas constant [J/mol/K]


mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

% create empty matrices
rhow  = zeros(size(T));
kappa  = zeros(size(T));
phi  = zeros(size(T));

% call external function to calculate water density and compressibility
% factor
[rho_w , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~, kappa0]  = eos_sw(press,temp,0);

% call external function to calculate gas phase fugacity coefficient
[~, ~, ~, ~, ~, ~, phi0] = eos_co2(press,temp);


% enlarge arrays
for i = 1:length(sal)
    rhow(:,:,i) = rho_w(:,:);
    kappa(:,:,i) = kappa0(:,:);
    phi(:,:,i) = phi0(:,:);
end

V10 = 1./rhow./55.51; % molar volume of water [m^3/mol]
f = pp.*phi; % gas phase fugacity of solute


% call function to calculate Henry constant
KHx = henry_CO2(T); % [1/MPa], KHx = x/f


%------------------------------------------------------------------------
% Calculation of second virial coefficients
%------------------------------------------------------------------------
B11 = 2.*pi.*Na.*sigma11.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek11./T)-1));
B12 = 2.*pi.*Na.*sigma12.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek12./T)-1));

%------------------------------------------------------------------------
% Calculation of solute infinite dilution partial molar volume [m^3/mol]
% No salt effects included!
%------------------------------------------------------------------------

V20 = N.*V10 + kappa.*R.*T.*(1 - N) + kappa.*R.*T.*rhow.*(2.*Omega.*...
    (B12 - N.*B11).*exp(-c1.*rhow) + (a./T.^5 + b).*(exp(c2.*rhow) - 1));

%------------------------------------------------------------------------
% Calculation of activity coefficient
%------------------------------------------------------------------------
[lambdaNa] = par2(P*10,T);
[zetaNaCl] = par4(P*10,T);

gamma =  exp(2.*lambdaNa.*(mNa + mK + 2.*mCa + 2.*mMg) ...% molalities in mol/kg
        - 0.07.*mSO4 + zetaNaCl.*(mNa + mK + 2.*mCa + 2.*mMg).*(mCl)) ;
%------------------------------------------------------------------------
% Calculation of pressure correction
%------------------------------------------------------------------------

% pressure correction using the relation:
%  int(V_mol/R/T dp)
%  where V_mol = partial molar volume of solvent, pressure in [Pa]

IntVo2 = V20.*(P-psh2o).*1d6;   % integrated molar volume of dissolved O2

pcor = exp(IntVo2./R./T);  %pressure correction factor

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
KHx = KHx./gamma./pcor; % pressure and salinty correction for Henry constant

% convert Henry constant to [mol/kg/MPa]
KH = 55.5*KHx./(1 - f.*KHx);

m = f.*KH; % [mol/kg]

end

%------------------------------------------------------------------------

function KHCO2 = henry_CO2(T)
% calculation of Henry constant

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

Tc = 647.096;       % critical temperature of water
Pc = 22.064;        % critical pressure of water
tau = 1 - T./Tc;    % dimensionless temperature
Ts = T./Tc;         % reduced temperature

%------------------------------------------------------------------------
% Calculation of water vapor pressure
%------------------------------------------------------------------------

a1 = -7.85951783;
a2 = 1.84408259;
a3 = -11.7866497;
a4 = 22.6807411;
a5 = -15.9618719;
a6 = 1.80122502;

% vapor pressure of water
pvap = Pc.*exp(Tc./T.*(a1.*tau + a2.*tau.^1.5 + a3.*tau.^3 + a4.*tau.^3.5...
    + a5.*tau.^4 + a6.*tau.^7.5));

%------------------------------------------------------------------------
% Calculation of Henry constant
%------------------------------------------------------------------------

A = -9.4234;
B = 4.0087;
C = 10.3199;

% Henry constant in [MPa]
kh_H = exp(log(pvap) + A./Ts + B.*(1 - Ts).^0.355 ./Ts + ...
    C.*exp(1 - Ts).*Ts.^(-0.41));

KHCO2 = 1./kh_H;%  Henry constant in [1/MPa]


% ----------------------------------------------------------

function [lambdaNa] = par2(P,T)
% parametric calculation of interaction parameter lambda

c1 = -0.411370585;
c2 = 0.000607632013;
c3 = 97.5347708;
c8 = -0.0237622469;
c9 = 0.0170656236;
c11 = 0.0000141335834;

lambdaNa = c1 + c2.*T + c3./T + c8.*P./T + c9.*P./(630 - T)...
     + c11.*T.*log(P);


% ----------------------------------------------------------

function [zetaNaCl] = par4(P,T)
% parametric calculation of interaction parameter zeta

c1 = 0.000336389723;
c2 = -0.0000198298980;
c8 = 0.0021222083;
c9 = -0.00524873303;

zetaNaCl = c1 + c2.*T + c8.*P./T + c9.*P./(630 - T);

% ----------------------------------------------------------
% ------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc);   % pressure in[MPa]

% ------------------------------------------------------------
