function [m, KH, KHx] = soluhenry_o2sw(press,temp,sal,pp)

% SOLUHENRY_O2SW -  calculates the equilibrium concentration of O2
%     in seawater from Henry's law
%   
%     Usage:  [m KH KHx] = soluhenry_o2sw(p,T,S,pp)
%
%     Input parameters (scalars or vectors, pp: scalar or matrix of size p x T x S): 
%           p        -  pressure [MPa]               (0.1 < p < 20)
%           T        -  temperature [degC]           (0 < T < 127)
%           S        -  practical salinity           (S < 350)
%           pp       -  partial pressure of O2 [MPa] (pp < p)
%                       pp is set to p if no input value is provided 
%     
%     Output parameter: 
%           m        - molality of O2 in seawater [mol/kg]
%           KH       - pressure and salinity corrected Henry constant [mol/kg/MPa]
%           KHx      - pressure and salinity corrected Henry constant [1/MPa]
%
%     References:
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of O2)
%     Geng & Duan (2010) GCA 74, 5631-5640. (gamma term)
%
%     Comments: 
%     Henry's law is not well suited for high pressures and should be used
%     with care at high pressures: Keep in mind that results are expected
%     to be inaccurate.
%
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_O2SW:Input','SOLUHENRY_O2SW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>81.8) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>120)
else
    bad = find(press<0.1 | press > 81.8);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_O2SW:OutOfParameterRange','SOLUHENRY_O2SW: Input values out of range!')
    warning('SOLUHENRY_O2SW:OutOfParameterRange','Equations valid for p = 0.1-81.8 MPa, T= 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press,temp+273.15,sal); % temperature in [K]

    if nargin == 3 % if no partial pressure is provided by user, partial pressure is set to system pressure
       pp = P; 
    end
% Oxygen specific constants
a = -2.d9;
b = 2.18d-3;
N = 0.98;
sigma12 = 3.01d-8; % [cm]
ek12 = 453;

lambdaNa = 1.9997d-1;
lambdaK  = 1.5022d-1;
xiNaCl = -1.2793d-2;
lambdaMg = 3.1715d-1;
lambdaCa = 3.5135d-1;
lambdaSO4 = 1.4383d-1;

Omega = 55.51d-6; % 
c1 = 0.0033; % [m^3/kg]
c2 = 0.002; %[m^3/kg]
sigma11 = 2.561d-8; % [cm]
ek11 = 1255;
lambda = 1.22;
Na = 6.02215d23; % Avogadro's number
R = 8.3145; % gas constant [J/mol/K]

mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

% create empty matrices
rhow  = zeros(size(T));
kappa  = zeros(size(T));
phi  = zeros(size(T));

% call external function to calculate water density and compressibility
% factor
[rho_w , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~, kappa0]  = eos_sw(press,temp,0);

% call external function to calculate gas phase fugacity coefficient
[~, ~, ~, ~, ~, ~, phi0] = eos_o2(press,temp);


% enlarge arrays
for i = 1:length(sal)
    rhow(:,:,i) = rho_w(:,:);
    kappa(:,:,i) = kappa0(:,:);
    phi(:,:,i) = phi0(:,:);
end

V10 = 1./rhow./55.51; % molar volume of water [m^3/mol]
f = pp.*phi; % gas phase fugacity of solute

% call function to calculate Henry constant
KHx = henry_o2(T); % [1/MPa], KHx = x/f

%------------------------------------------------------------------------
% Calculation of second virial coefficients
%------------------------------------------------------------------------
B11 = 2.*pi.*Na.*sigma11.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek11./T)-1));
B12 = 2.*pi.*Na.*sigma12.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek12./T)-1));

%------------------------------------------------------------------------
% Calculation of solute infinite dilution partial molar volume [m^3/mol]
% No salt effects included!
%------------------------------------------------------------------------

V20 = N.*V10 + kappa.*R.*T.*(1 - N) + kappa.*R.*T.*rhow.*(2.*Omega.*...
    (B12 - N.*B11).*exp(-c1.*rhow) + (a./T.^5 + b).*(exp(c2.*rhow) - 1));

%------------------------------------------------------------------------
% Calculation of activity coefficient
%------------------------------------------------------------------------
gamma = exp(  2.*lambdaNa.*mNa + 2.*lambdaK.*mK ...
    + 4.*lambdaCa.*mCa + 4.*lambdaMg.*mMg...
    + xiNaCl.*(mNa + mK + 2*mCa + 2*mMg).*(mCl + 2*mSO4) ...
    + 4.*lambdaSO4.*mSO4);
%------------------------------------------------------------------------
% Calculation of pressure correction
%------------------------------------------------------------------------

% pressure correction using the relation:
%  int(V_mol/R/T dp)
%  where V_mol = partial molar volume of solvent, pressure in [Pa]

IntVo2 = V20.*(P-psh2o).*1d6;   % integrated molar volume of dissolved O2

pcor = exp(IntVo2./R./T);  %pressure correction factor

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
KHx = KHx./gamma./pcor; % pressure and salinty correction for Henry constant

% convert Henry constant to [mol/kg/MPa]
KH = 55.5*KHx./(1 - f.*KHx);

m = f.*KH; % [mol/kg]
end

% ------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc);   % pressure in[MPa]

%------------------------------------------------------------------------
function KHo2 = henry_o2(T)
% calculation of Henry constant

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

Tc = 647.096;       % critical temperature of water
Pc = 22.064;        % critical pressure of water
tau = 1 - T./Tc;    % dimensionless temperature
Ts = T./Tc;         % reduced temperature

%------------------------------------------------------------------------
% Calculation of water vapor pressure
%------------------------------------------------------------------------

a1 = -7.85951783;
a2 = 1.84408259;
a3 = -11.7866497;
a4 = 22.6807411;
a5 = -15.9618719;
a6 = 1.80122502;

% vapor pressure of water
pvap = Pc.*exp(Tc./T.*(a1.*tau + a2.*tau.^1.5 + a3.*tau.^3 + a4.*tau.^3.5...
    + a5.*tau.^4 + a6.*tau.^7.5));

%------------------------------------------------------------------------
% Calculation of Henry constant
%------------------------------------------------------------------------

A = -9.4025;
B =  4.4923;
C = 11.3387;

% Henry constant in [MPa]
kh_H = exp(log(pvap) + A./Ts + B.*(1 - Ts).^0.355 ./Ts + ...
    C.*exp(1 - Ts).*Ts.^(-0.41));

KHo2 = 1./kh_H;%  Henry constant in [1/MPa]
