function  sw_phasediagram(S,T1,T2)

% SW_PHASEDIAGRAM -  plots the phasediagrams of the binary system NaCl-H2O
%     for salinities up to 1000 g/kg, temperatures up to 1000 degC, and
%     pressures of >500 MPa as calculated by phase_sw.
%
%     Usage: sw_phasediagram(S,T1,T2)
%
%     Input parameters (scalar):
%           S   - practical salinity (default: 300)
%           T1  - start of temperature range [degC] (default is 0 degC)
%           T2  - end of temperature range [degC]   (default is 800 degC)
%
%     References:
%     Driesner & Heinrich (2007) GCA 71, 4880-4901.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               IFM-GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------
 
% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters, default value: S = 300 g/kg,
%  T = (T_Vapor_Liquid_Halite - 50) - T_crit [C]
%------------------------------------------------------------------------


if nargin == 0 
    S = 300;
end


%------------------------------------------------------------------------
% Define temperature range
%------------------------------------------------------------------------

[~, ~, Tcrit , ~, T_VLH] = phase_sw(100,S); %  find critical and 3-phase point 
   
       
if nargin < 2 
       if ~isnan(T_VLH) && ~isnan(Tcrit)
             temp1 = ((T_VLH-50):1:Tcrit);
       elseif ~isnan(T_VLH)
           temp1 = ((T_VLH-50):1:800);
       elseif ~isnan(Tcrit)
           temp1 = (1:1:Tcrit);
       else 
           temp1 = (0:1:800);
       end
else 
     temp1 = (T1:1:T2);
end
       
%------------------------------------------------------------------------
% Calculation of  phase boundaries
%------------------------------------------------------------------------

[P_sat, P_VL, Tcrit, Pcrit, T_VLH, P_VLH] = phase_sw(temp1,S); %  [MPa]
     

  
%------------------------------------------------------------------------
% Plot phase diagram 
%------------------------------------------------------------------------      
if isnan(Tcrit)
      figure(1)
       clf
       if nargin >=3
        set(gca,'XLim',[T1 T2]);
       end
       if nargin == 5
            set(gca,'YLim',[P1 P2]); 
       end
       axis ij
       hold on;
       plot(temp1,P_sat,'g-','LineWidth',2);
       plot(temp1,P_VL,'b-','LineWidth',2);
       %set(gca,'YLim',[0 p2]);
       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
       ylabel('Pressure / MPa','FontSize',12,'FontWeight','bold');
       title(['Phase diagram of seawater, S =' , num2str(S)],'FontSize',14,'FontWeight','bold');
       plot(T_VLH,P_VLH,'k.','MarkerSize',20);
        legend('saturation curve','vapor-liquid equlibrium curve','vapor-liquid-halite equilibrium','Location','S')
       hold off; 
else

       figure(1)
       clf
       if nargin >=3
        set(gca,'XLim',[T1 T2]);
       end
       if nargin == 5
            set(gca,'YLim',[P1 P2]); 
       end
       axis ij
       hold on;
       plot(temp1,P_sat,'g-','LineWidth',2);
       plot(temp1,P_VL,'b-','LineWidth',2);
       %set(gca,'YLim',[0 p2]);
       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
       ylabel('Pressure / MPa','FontSize',12,'FontWeight','bold');
       title(['Phase diagram of seawater, S =' , num2str(S)],'FontSize',14,'FontWeight','bold');
       plot(Tcrit,Pcrit,'r.','MarkerSize',20);
       plot(T_VLH,P_VLH,'k.','MarkerSize',20);
        legend('saturation curve','vapor-liquid equlibrium curve','critical point','vapor-liquid-halite equilibrium','Location','S')
       hold off;
end
 