function eta = visco_n2(press,temp)

% VISCO_N2 - calculates the dynamic viscosity of N2 from a
%      multiparametric equation
%
%      Usage: eta = visco_n2(p,T)
%
%      Input parameters (scalars or vectors): 
%           p        -  pressure [MPa] (0.1-100 MPa), 
%           T        -  temperature [degC] ((-120) - 727 degC) 
%
%      Output parameter: 
%           eta      - dynamic viscosity [Pa*s]
%
%      References:
%      Lemmon & Jacobsen (2004) Int.J.Thermophys 25(1), 21-69.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('VISCO_N2:Input','VISCO_N2: Missing input parameter! Please provide p and T!')
else


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) &&  ~any( temp<(-120) | temp>727)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(-120) | temp>727);
    temp(bad)=NaN*ones(size(bad));
    warning('VISCO_N2:OutOfParameterRange','VISCO_N2: Input values out of range!')
    warning('VISCO_N2:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=(-120)-727 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~,T] = ndgrid(press,temp+273.15); % P [MPa], T [K]

Tc = 126.192;   % critical temperature [K]
rhoc = 11.1839; % critical molarity [mol/dm^3]
% pc = 3.3958;    % critical pressure [MPa]
M = 28.01348;   % molar weight [g/mol]
ek = 98.94;     % Lennard Jones energy parameter epsilon/k [K]
sigma = 0.3656; % Lennard Jones parameter sigma [nm]

% call external function to calculate density of CO2
rho = eos_n2(press,temp); %[kg/m^3]
rho = rho/M; % [mol/dm^3]

tau = Tc./T;
Ts = T./ek;
delta = rho./rhoc;

b(1) = 0.431;
b(2) =-0.4623;
b(3) = 0.08406;
b(4) = 0.005341;
b(5) =-0.00331;

N(1) = 10.72;
N(2) =  0.03989;
N(3) =  0.001208;
N(4) = -7.402;
N(5) =  4.620;

t(1) = 0.1;
t(2) = 0.25;
t(3) = 3.2;
t(4) = 0.9;
t(5) = 0.3;

d(1) = 2;
d(2) = 10;
d(3) = 12;
d(4) = 2;
d(5) = 1;

l(1) = 0;
l(2) = 1;
l(3) = 1;
l(4) = 2;
l(5) = 3;

gamma(1) = 0;
gamma(2) = 1;
gamma(3) = 1;
gamma(4) = 1;
gamma(5) = 1;

%------------------------------------------------------------------------
% Calculation of the residual viscosity etar
%------------------------------------------------------------------------
etar = zeros(size(T));

for i = 1:5
    etar = etar + N(i).*tau.^t(i).*delta.^d(i).*exp(-gamma(i).*delta.^l(i));
end 

%------------------------------------------------------------------------
% Calculation of the dilute gas viscosity eta0
%------------------------------------------------------------------------

Omega = exp(b(1)+ b(2).*(log(Ts)) + b(3).*(log(Ts)).^2+ ...
    b(4).*(log(Ts)).^3+ b(5).*(log(Ts)).^4 );

eta0 = 0.0266958.*sqrt(M.*T)./(sigma.^2.*Omega);

%------------------------------------------------------------------------
% Calculation of the viscosity eta
%------------------------------------------------------------------------

eta = eta0 + etar; % [Pa s]
eta = eta*1d-6;% [Pa s]

end